/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2019
 */
Polymer({

	is: 'maximo-inspector-complete',

	behaviors: [BaseComponent, Inspections, Polymer.IronA11yKeysBehavior],

	listeners: {
	},
  	
  properties: {
    
    /**
     * Inspection Result List 
     */
    resultList: {
      type: Array,
      value: function() {
        return [];
      },
      observer: 'resultListChanged'
    },

    /**
     * processed inspection Result List
     */
    processedInspResultList: {
      type: Array,
      value: function() {
        return [];
      },
      readOnly: true
    },

    /**
     * Allowed Action List
     */
    allowedActionList: {
      type: Array,
      value: function() {
        return [];
      },
      observer: 'allowedActionListChanged'
    },

    /**
     * Tooltip Action List
     */
    actionList: {
      type: Array,
      value: function() {
        return [];
      }
    },

    /**
     * Common actions available for all inspections 
     */
    batchActionsList: {
      type: Array,
      value: function() {
        return [];
      }
    },

    /**
     * Stores inspection result reference of the action to be taken
     */
    inspectionActionReference: {
      type: Object,
      value: function() {
        return null;
      }
    },

    /**
     * Set event target for keysbehavior
     */
    keyEventTarget: {
      type: Object,
      value: function() {
        return document.body;
      }
    },
    
    /**
     * Set isToggleablePanel 
     */
    isToggleablePanel: {
		type: Boolean,
		value: true,
		observer: '_isToggleablePanelChanged'
	},
    
  },

  keyBindings: {
    'esc:keyup': '_escPressed'
  },

  /**
   * Observer resultList
   */
  resultListChanged: function(changeRecord) {
    if (changeRecord) {
      var computedResultList = this._computeInspResultList(changeRecord);
      this._setProcessedInspResultList(computedResultList);
      this._loadInspectionsAllowedActions();
    }

  },

  /**
   * Observer allowed actions list
   */
  allowedActionListChanged: function(changeRecord) {
    this.batchActionsList = this._computeBatchActions(this.resultList);
  },

  /**
   * Computed property
   */
  _computeBatchActions: function(inspResultList) {

    var commonActions = this._getCommonInspActions(inspResultList);
    if (commonActions.length) {
      this.$.batchActions.removeAttribute('disabled'); 
      this.$.batchActions.removeAttribute('hidden');
    } else {
      this.$.batchActions.setAttribute('disabled', 'disabled');
      this.$.batchActions.setAttribute('hidden', true);
    }
    return commonActions;
  },

  /**
   * Computed property
   */
  _computeInspResultList: function(inspResultList) {

    var processedList = [];
    // Iterate over list
    for (var inspResult of inspResultList) {

      var inspForm = inspResult.inspectionform[0];
      //var procInspResult = this.buildLightweightResult(inspResult);
      var procInspResult = inspResult;

      // Look for displaymessage != null in inspfieldresult array
      var filteredFieldResultList = this.findProcessedResponses(inspResult.inspfieldresult);

      var processedFieldList = [];
      // Build question summary
      for (var actionFielResult of filteredFieldResultList) {
        // Find its question
        var questions = inspForm.inspquestion.filter(function(question){
          return question.inspquestionnum === actionFielResult.inspquestionnum;
        });
        var question = questions.length === 1 ? questions[0] : null;
        if (!question || !question.inspfield) {
          console.error('Unable to match question');
          continue;
        }

        // Find its field
        var fields = question.inspfield.filter(function(field){
          return field.inspfieldnum === actionFielResult.inspfieldnum;
        });
        var field = fields.length === 1 ? fields[0] : null;
        if (!field) {
          console.error('Unable to match field');
          continue;
        }

        // Build string for display (buildQuestionSummary)
        var responseType = this.getResponseFieldType(field.fieldtype_maxvalue);
        var response = actionFielResult[responseType];
        var questionSummary = this.buildQuestionSummary(question, field);

        actionFielResult._questionSummary = questionSummary;
        actionFielResult._questionResponse = response;
        processedFieldList.push(actionFielResult);
      }
      
      procInspResult._procInspfieldResult = processedFieldList;
      processedList.push(procInspResult);
    }

    return processedList;
  },

  /**
   * Load allowed actions for set of inspections result
   */
  _loadInspectionsAllowedActions: function(){

    //Activate spinner
    $M.toggleWait(true);

    var inspresultcollection = this.$.inspresultactionscollection;

    //Fetch records
    inspresultcollection.refreshRecords();

  },

  /**
   * Handler mx-collection inspresultactionscollection success
   * Request allowed actions to all inspection result
   */
  _actionsRefreshSuccessHandler: function(event) {
    $M.toggleWait(false);
    // Build allowed actions
    var records = this.$.inspresultactionscollection.collectionData;
    var procAllowedActions = [];
     if (records &&
         records.length && 
         'allowedactions' in records[0] && 
         Object.keys(records[0].allowedactions).length) {
       procAllowedActions = Object.values(records[0].allowedactions);
     }

     this.allowedActionList = procAllowedActions;
  },

  /**
   * Handler mx-collection inspresultactionscollection error
   * Request allowed actions to all inspection result
   */
  _actionsRefreshErrorHandler: function(event) {
    $M.toggleWait(false);
    console.warn('problems to fetch allowed actions');
    this.allowedActionList = [];
  },
  
  /**
   * Executes action for all records from the list in a linear way to avoid 
   * conflict updating the same record. IJ35922
   * @param {string} formScript 
   * @param {string} actionName 
   * @returns {Promise} Resolves when all actions succeeds. Rejects when one of them fails.
   */
   executeActionInBatch: async function(formScript, actionName) {

    return new Promise(async (resolve, reject) => {
      // let errors = [];
      let responses = [];
      for (var inspResult of this.resultList) {
        if (this.existScript(inspResult, formScript)) {
          this.$.inspResultResource.resourceUri = inspResult.href;
          try {
            const response = await this.$.inspResultResource.updateAction(actionName);
            responses.push(response);
          } catch(error) {
            // errors.push(error);
            reject(error);
          }
        }
      }
      resolve(responses);
    })
  },

  /**
   * Handle batch action click
   * Request action to all inspection result
   */
  _takeAction: function(event) {

    // Get model the request came from to identify which action to request
    var formScript = event.model.action;
    var actionName = this.parseActionName(formScript.autoscript);
    var self = this;

    var responseAction;
    $M.toggleWait(true);

    // Checks if it is batch
    if (!this.inspectionActionReference) {
      responseAction = this.executeActionInBatch(formScript, actionName);
    } else {
      this.$.inspResultResource.resourceUri = this.inspectionActionReference.href;
      responseAction = this.$.inspResultResource.updateAction(actionName);
    }

    responseAction.then(function(values) {
      const valuesArray = Array.isArray(values) ? values : [values];
      // Replace inspection result with response
      for (var response of valuesArray) {
        self.replaceInspResult(response.response);
      }
      // Manually call property computer
      var computedResultList = self._computeInspResultList(self.resultList);
      self._setProcessedInspResultList(computedResultList);
      
      // Success label
      $M.alert($M.localize('uitext','mxapiinspresult','action_executed'), $M.alerts.info);

    }).catch(function (reason) {

      var errorMsg = reason.request?.response?.Error?.message || reason;
      $M.alert(self.escapeHtml(errorMsg), $M.alerts.error);

    }).finally(() => {

      $M.toggleWait(false);
    });

  },

  /**
   * Replace inspection result object with autoscript response
   */
  replaceInspResult: function(newInspResult){
    // Find inspection result in resultList
    var indexFound = this.resultList.findIndex(function(resultItem){
      return resultItem.inspectionresultid === newInspResult.inspectionresultid;
    });
    // Replace inspection result with new
    this.splice('resultList', indexFound, 1, newInspResult);
  },
  
  /**
   * Batch actions button clicked
   * Open tooltip with common scripts
   */
  _batchActions: function(event) {
    
    // Fetch common actions from insp result
    var actions = JSON.parse(JSON.stringify(this.batchActionsList));
    this.set('actionList', actions);

    this.set('inspectionActionReference', null);

    var tooltip = this.$.batchActionsTooltip;
    var content = this.$.batchActionsToolTipContainer;
    tooltip.show(content, event.currentTarget);
    // subtract host offset
    var top = tooltip.elementOffset.top - $j(this).offset().top + 50;
    $j(tooltip.$.tooltip).css({'top':top+'px'});    
//    tooltip.elementOffset.top -= $j(this).offset().top;
//    tooltip._positionTooltip('east');
  },

  /**
   * Item actions button clicked
   * Open tooltip with insp result scripts
   */
  _showItemAction: function(event) {

    var inspResult = event.model.resultItem;
    // Fetch actions from this particular insp result
    var actions = this._getInspActions(inspResult);
    this.set('actionList', actions);
    
    this.set('inspectionActionReference', inspResult);
    
    var tooltip = this.$.batchActionsTooltip;
    var content = this.$.batchActionsToolTipContainer;
    tooltip.show(content, event.currentTarget);
    // subtract host offset
    var top = tooltip.elementOffset.top - $j(this).offset().top + 40;
    $j(tooltip.$.tooltip).css({'top':top+'px'});

  },

	/**
	 * Exit button clicked
	 * Close modal
	 */
  _exitCompletion: function(e) {
    this.closeMe();
	},
	
	/**
	 * esc key handler
	 * Close modal
	 */
  _escPressed: function() {
    this.closeMe();
  },

  /**
   * Extract list of scripts on inspection result
   */
  _getInspActions: function(inspResult) {

    var form = (!inspResult) ? 
        null : (!inspResult.inspectionform) ? 
            null : (!inspResult.inspectionform.length) ?
                null : inspResult.inspectionform[0];

    var tmpList = (!form) ?
        [] : (!form.inspformscript) ?
            [] : form.inspformscript;

    var list = [];
    // Match list of script with allowed actions
    for (var i = 0; i < tmpList.length; i++) {
      var script = tmpList[i];
      var filter = this.allowedActionList.filter(function(allowedAction){
        if (allowedAction.impl === script.autoscript) {
          if (!('description' in script)) {
            script.description = allowedAction.description;
          }
          return true;
        } else {
          return false;
        }
      });
      if (filter.length) {
        list.push(script);
      }
    }
    return list;
  },

  /**
   * Extract list of common scripts of list of inspection results
   */
  _getCommonInspActions: function(resultList) {

    var commonActions = [];
    if (resultList.length) {
      // Grab the actions of the first inspresult case there's only one
      var tmpCommonActions = this._getInspActions(resultList[0]);

      if (resultList.length > 1) {
        
        for (var i = 1; i < resultList.length; i++) {
          var currentResultActions = this._getInspActions(resultList[i]);
          // Filter common scripts
          tmpCommonActions = tmpCommonActions.filter( function(n) {
            for (var script of currentResultActions) {
              if (script.autoscript === n.autoscript) {
                return true;
              }
            }
            return false;
          });
        }
        
      }
      commonActions = tmpCommonActions;
    }
    
    return commonActions;
  },

  /**
   * Close modal host
   */
  closeMe: function() {
    this.fire('cancel');
  },

  /**
   * Check existence of message to display in inspection instance
   */
  hasMessage: function(inspection) {
    return inspection.displaymessage ? true : false;
  },

  /**
   * Check existence of actions to display in inspection instance
   */
  hasAction: function(inspection) {
    var actions = this._getInspActions(inspection);
    return Boolean(actions.length);
  },

  /**
   * Return title from inspection 
   */
  getTitle: function(inspection){
    
    return (!inspection) ? 
        '' : (!inspection.inspectionform) ? 
            '' : (!inspection.inspectionform.length) ?    
                '' : (!inspection.inspectionform[0].name) ? 
                    '' : inspection.inspectionform[0].name;
  },

  /**
   * Return subtitle from inspection
   */
  getSubtitle: function(inspection){
    return this.getElementDescription(inspection);
  },
  
  /**
   * Build completion statement
   */
  getCompletion: function(inspection) {
    var statusList = inspection.inspresultstatus;
    var completion = '';
    if (statusList) {
      for (var i = statusList.length-1; i >= 0; i--) {
        if (statusList[i].status_maxvalue === 'COMPLETED') {
          var lastCompletion = statusList[i];
          completion = $M.localize('uitext','mxapiinspresult','inspections_complete_on_by', [lastCompletion.changedate_localized, lastCompletion.changeby]);
          break;
        }
      }
    }
    return completion;
  },

  /**
   * Get name used to call script action
   */
  parseActionName: function(scriptName) {
//    var regex = /\.*(\w+$)/g;
//    var match;
    var action;
//    if ((match = regex.exec(scriptName)) !== null) {
//      action = match[1];
//    }
    var arrayOfStr = scriptName.split('.');
    if (arrayOfStr.length > 1) {
      action = arrayOfStr[arrayOfStr.length-1];
    } else {
      console.warn('Unable to match dot pattern in automation script name');
      action = scriptName;
    }

    return action;
  },

  /**
   * Build a string that summarize question, field and field response
   */
  buildQuestionSummary: function(question, field, rawResponse) {
    var questionSummary = question.sequence + ' - ' + question.description;
    return questionSummary;
  },

  /**
   * Find field responses processed by autoscripts
   */
  findProcessedResponses: function(fieldResultList) {
    var filteredFieldResultList = [];
    if (fieldResultList) {
      filteredFieldResultList = fieldResultList.filter(function(elem){
        // displaymessage / fupobjectid / fupobject
        return elem.displaymessage && elem.displaymessage.length;
      });
    }
    return filteredFieldResultList;
  },

  /**
   * Check if script exists in inspection result
   */
  existScript: function(inspResult, formScript) {

    var list = this._getInspActions(inspResult);
    var filteredList = list.filter(function(script){
      return formScript.autoscript === script.autoscript;
    });
    
    return Boolean(filteredList.length);
  },
   
  /**
   * Observer isToggleablePanel
   */
  _isToggleablePanelChanged: function(newIsToggleableValue) {

//      this.toggleClass('main-page-mobile', this.isToggleablePanel, this.$.mainPageDiv);
//	  this.toggleClass('footer-mobile', this.isToggleablePanel, this.$.footer);

  },
  
   
});
