/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2019
 */
Polymer({
	is: 'maximo-inspector-create',
	
  	behaviors: [BaseComponent, ResizeBehavior],
  	
  	listeners: {
  		'inspections-screen-resize' : '_screenResize',
  	},
  	
    properties: {
    	
    	/**
    	 * List of inspection form
    	 */
    	dataSet: {
    		type: Array
    	},
    	
    	/**
    	 * 
    	 */
    	selectedTile: {
    		type: Object    	
    	},
    	
    	/**
    	 * Indicator to show/hide message
    	 */
    	hasFirstStep: {
    		type: Boolean,
    		value: false,
    		readOnly: true
    	},
    	
    	/**
    	 * Flag to hide chevron and input 
    	 */
    	hideInput: {
    		type: Boolean,
    		value: true,
    		readOnly: true
    	},
    	
    	/**
    	 * Placeholder for typeahead input
    	 */
    	inputPlaceholder: {
    		type: String,
    		value: ''
    	},
    	
    	/**
    	 * Type ahead value
    	 */
    	inputValue: {
    		type: String,
    		value: ''
    	},
    	
    	/**
    	 * Search terms used for both collections
    	 */
    	searchTerm: {
    		type: String,
    		value: ''
    	},
    	
    	/**
    	 * Indicator to show/hide loading   
    	 */
    	showLoading: {
    		type: Boolean,
    		value: false
    	},
    	
    	
    	/**
    	 * Flag to to indicate if there's type ahead result
    	 */
    	hasResult: {
    		type: Boolean,
    		value: false
    	},
    	
    	/**
    	 * Flag to indicate if there's inspection form
    	 */
    	hasRecord: {
    		type: Boolean,
    		value: false,
    		readOnly: true
    	},
    	    	
    	/**
    	 * collection used to search type ahead
    	 */
    	selectedCollection: {
    		type: Object,
    		value: null
    	},
    	
    	/**
    	 * type ahead result set
    	 */
    	typeAheadResultSet: {
    		type: Array,
    		value: []
    	},
    	
    	/**
    	 * Selected element
    	 */
    	selectedElement: {
    		type: Object
    	},
    	    	    	
    	/**
    	 * Set Selected Asset from type ahead
    	 */
    	selectedAsset : {
    		type: String
    	},
    	
    	/**
    	 * Set Selected Location from type ahead
    	 */
    	selectedLocation : {
    		type: String
    	},
 
    	/**
    	 * Set Selected Asset/Location Siteid from type ahead
    	 */
    	selectedSiteid : {
    		type: String
    	},
    	
    	/**
    	 * Set Selected Reference Object
    	 */
    	selectedReferenceObject : {
    		type: String
    	},
    	
		/**
		 * Inspection Org id
		 */
		inspOrgid: {
			type: String
		},
		/**
    	 * Search value used for Toggleing
    	 */
    	defaultSearchValue: {
    		type: String,
    		value: ''
    	},
    	
	},
	
	/**
	 * window resize listener
	 */
	_screenResize: function(ev) {
		ev.stopPropagation();

		const classToggler = this.deviceType === deviceTypes_.PHONE 
			|| (this.deviceType === deviceTypes_.TABLET 
					&& this.orientation === orientationTypes_.PORTRAIT);

		this.toggleClass('mobile', classToggler, this.$.inspectorCreateMainDiv);
	},

	/**
	 * Computed property
	 */
	_computeNoInspectionAbsence: function(hasFirstStep, hasRecord) {
		return (!hasFirstStep || (hasFirstStep && hasRecord));
	},

	dataSetRefresh: function () {
				
		var total;
		if (!this.dataSet) {
			total = 0;
		}else {
			total = this.dataSet.length;
		}
		
		this._resultCounter = $M.localize('uitext', 'mxapibase', 'countresults',[total]);
		
		if (total === 0){
			this._setHasRecord(false);
		}else{
			this._setHasRecord(true);
		}
	},
	
	/**
	 * Listens to asset button click
	 */
	assetList: function(e) {
		this.selectedReferenceObject = 'ASSET';
		this._setHideInput(false);
		this.inputPlaceholder = $M.localize('uitext', 'mxapiinspresult','asset_search_placeholder');
		this.selectedCollection = this.$.assetCollection;
		
		this.clearFirstStep();
		
		$j(this.$.inspectorLocationButton).toggleClass('assetLocationButtonActive',false);
		$j(this.$.inspectorAssetButton).toggleClass('assetLocationButtonActive',true);
		
		//this.fire('refreshFormCollection');
		this.setSearchBarFocus();
	},
	
	/**
	 * Listens to location button click
	 */
	locationList: function(e) {
		this.selectedReferenceObject = 'LOCATION';
		this._setHideInput(false);
		this.inputPlaceholder = $M.localize('uitext', 'mxapiinspresult','location_search_placeholder');
		this.selectedCollection = this.$.locationCollection;
		
		this.clearFirstStep();
		
		$j(this.$.inspectorLocationButton).toggleClass('assetLocationButtonActive',true);
		$j(this.$.inspectorAssetButton).toggleClass('assetLocationButtonActive',false);
		
		//this.fire('refreshFormCollection');
		this.setSearchBarFocus();
	},
	
	/**
	 * Refresh collection based on type 
	 */
	searchAgain: function (e) {
		
		//console.log('Listens to input: ' + this.inputValue);

		var input = e.currentTarget;
		
		if (!this.inputValue || this.inputValue.length < 3){
			$j(this.$.itemList).css({'display':'none'});
			if(input.keyupTimer){
				window.clearTimeout(input.keyupTimer);
			}
			this._setHasFirstStep(false);
			return;
		}else{
			this.showLoading=true;
			$j(this.$.itemList).css({'display':'block'});
		}
		
		if(input.keyupTimer){
			window.clearTimeout(input.keyupTimer);
		}
		
		var that = this;
	    input.keyupTimer = setTimeout(function(){
	    	that.performSearch();
		}, 1000);
		
	},
	
	/**
	 * Clear first step also hides input
	 */
	clearForm: function() {
		this.clearFirstStep();
		this._setHideInput(true);
	},
	
	
	/**
	 * Clear first step variables, inputs, selections
	 */
	clearFirstStep: function() {
		this._setHasFirstStep(false);
		this.selectedElement = null;
		this.typeAheadResultSet = [];
		$j(this.$.itemList).css({'display':'none'});
		
		this.inputValue = '';
		$j(this.$.inspectorLocationButton).toggleClass('assetLocationButtonActive',false);
		$j(this.$.inspectorAssetButton).toggleClass('assetLocationButtonActive',false);
	},
	
	/**
	 * Prepare conditions and trigger a collection refresh
	 */
	performSearch: function () {
		
		// IJ35027 - For searching terms, using the """ to a Literal Search, including spaces, but removing 
		// the spaces in the beginning and in the end of the search string
		this.searchTerms = '"' + '%' + (this.inputValue? this.inputValue.trim():"")+ '%' + '"';

		var orgId = $M.getMaxauth().whoami.defaultOrg;
		var siteId = ($M.getMaxauth().whoami.defaultSite) ? $M.getMaxauth().whoami.defaultSite : '';
		
		var maximoQbe = new MaximoQbe();
		maximoQbe.setEquals('orgid',orgId).setEquals('siteid',siteId);
		var qbeObject = maximoQbe.toJSON();
		
		this.selectedCollection.queryWhere = qbeObject;

		//Org filter
//		if (this.inspOrgid) {
//			this.selectedCollection.keySearchAttributeName = 'orgid';
//			this.selectedCollection.keySearch = this.inspOrgid;
//		}
		
		this.selectedCollection.refreshRecords();
	},
	
	/**
	 * Asset collection success handler
	 */
	_handleAssetDataRefreshed: function(e) {
		$j(this.$.itemList).css({'display':'block'});
		
		this.typeAheadResultSet = this.selectedCollection.collectionData;
		if (this.typeAheadResultSet && this.typeAheadResultSet.length > 0) {
			this.hasResult = true;
		}else {
			this.hasResult = false;
		}

		//TODO set collection ref
		var that = this;
		setTimeout(function(){ that.showLoading = false; }, 500);
	},
	
	/**
	 * Asset collection fail handler
	 */
	_handleAssetDataError: function(e){
		this.showLoading = false;
		
		$j(this.$.itemList).css({'display':'block'});
		console.error('Problems to execute query');
	},
	
	/**
	 * Location collection success handler
	 */
	_handleLocationDataRefreshed: function() {
		$j(this.$.itemList).css({'display':'block'});
		
		this.typeAheadResultSet = this.selectedCollection.collectionData;
		if (this.typeAheadResultSet && this.typeAheadResultSet.length > 0) {
			this.hasResult = true;
		}else {
			this.hasResult = false;
		}

		//TODO set collection ref

		var that = this;
		setTimeout(function(){ that.showLoading = false; }, 500);
	},
	
	/**
	 * Location collection fail handler
	 */
	_handleLocationDataError: function(){
		this.showLoading = false;
		
		$j(this.$.itemList).css({'display':'block'});
		console.error('Problems to execute query');
	},
	
	/**
	 * Listens to type ahead option list
	 */
	_selectElement: function(e){
		
		this.dataHost.isGoingBack=false;
		//Store selected element in var
		this.selectedElement = e.model.elem;
		
		this.inputValue = this._getElemDesc(this.selectedElement);
		
		$j(this.$.itemList).css({'display':'none'});
		
		this._setHasFirstStep(true);
		this.async(function(){
			this.$.recommended.checked = true;
			this.requestRecommendedForms();
		});
	},
	
	/**
	 * Compute asset description
	 */
	_getElemDesc(item) {
		
		var curObjName = this.selectedCollection.objectName;
		
		if (curObjName === 'MXAPIASSET') {
			this.selectedReferenceObject='ASSET';
			this.selectedAsset=item.assetnum;
    	} else if (curObjName === 'MXAPIOPERLOC') {
    		this.selectedReferenceObject='LOCATION';
    		this.selectedLocation=item.location;
    	}
		
		this.selectedSiteid=item.siteid;
		return  item.description;
    },
    /**
     * Compute asset num
     */
    _getNum(item) {
    	
    	var curObjName = this.selectedCollection.objectName;
    	if (curObjName === 'MXAPIASSET') {
    		return  ' (' + item.assetnum + ')';
    	} else if (curObjName === 'MXAPIOPERLOC') {
    		return  ' (' + item.location + ')';
    	}
    },

    /**
     * Inspection tap handler
     */
    _formSelect : function(e){
        var record = e.model.inspForm;
        record.page = 'start';
        this.fire('processFormSelection',record);
    },
    
    /**
     * Input Text Focus
     */
    setSearchBarFocus: function(){
  		setTimeout(function(){
  			var inspectorInputTypeText = $j(this.$.inspectorInputTypeText).find('input');
  			inspectorInputTypeText.focus();
  		}.bind(this), 600);	
    },

    /**
     * Handler of recommended toggle button
     */
    _toggleForms: function(event) {
      var recommendedForms = event.detail;
      
      this.defaultSearchValue='';
      if($j(this).find('maximo-searchbar').length>0){
    	$j(this).find('maximo-searchbar')[0].$.inputval.value='';
      }
      this.dataHost.isGoingBack = false;
      this.$.inspectorCreateSearchBar.filterValue = '';
      if (recommendedForms) {
        this.requestRecommendedForms();
      } else {
    	this.requestAllForms();
      }
    },

    /**
     * Fetch inspection forms inside result set
     */
    joinRecommendedForms: function(resultSet) {
      var forms = [];
      resultSet.forEach(function(recomendation) {
        var tmpForms = recomendation.inspectionform;
        if (tmpForms) {
          forms = forms.concat(tmpForms);
        }
      });
      return forms;
    },

    /**
     * Submit request to maximo to 
     * fetch recommended forms based on reference item
     */
    requestRecommendedForms: function() {
      var collection = this.$.recommendedCollection;
	  var objectType = this.selectedReferenceObject;
	  var defaultOrg = ($M.getMaxauth().whoami.defaultOrg) ? $M.getMaxauth().whoami.defaultOrg : '';
      var objectId;
      
      collection.searchTermValue = this.defaultSearchValue;
      
      if (objectType === 'ASSET') {
        objectId = this.selectedAsset;
      } else if (objectType === 'LOCATION') {
        objectId = this.selectedLocation;
      }

      // Set filter
      var maximoQbe = new MaximoQbe();
			maximoQbe.setIn('objectname', objectType)
			         .setEquals('objectid', objectId)
				     .setEquals('orgid', defaultOrg);
      var qbeObject = maximoQbe.toJSON();
      collection.queryWhere = qbeObject;

      this.collectionRef = collection;

      //Activate spinner
      $M.toggleWait(true);

      // Refresh collection
      collection.refreshRecords();
    },

    /**
     * mx-collection recommendedCollection refreshRecords success handler
     */
    _recommendedFormsSuccessHandler: function(event) {
    	if(this.dataHost.isGoingBack===true){
			return;
		}
      this.dataSet = this.joinRecommendedForms(this.recommendedSet);
      this.dataSetRefresh();
      $M.toggleWait(false);
    },

    /**
     * mx-collection recommendedCollection refreshRecords error handler
     */
    _recommendedFormsErrorHandler: function(event ) {
      this.dataSet = [];
      this.dataSetRefresh();
      $M.toggleWait(false);
    },

    _hasFirstStepChanged: function(newV) {
      if (newV) {
        this.async(function(){
        }, 200);
      }
    },
    
    /**
     * Submit request to maximo to 
     * fetch All forms based on reference item
     */
    requestAllForms: function() {
			var collection = this.dataHost.$.inspformList;
			var defaultOrg = ($M.getMaxauth().whoami.defaultOrg) ? $M.getMaxauth().whoami.defaultOrg : '';
			
			collection.searchTermValue = this.defaultSearchValue;
			
			var maximoQbe = new MaximoQbe();
			maximoQbe.setEquals('orgid', defaultOrg);
			
            var qbeObject = maximoQbe.toJSON();
			collection.queryWhere = qbeObject;
			
			this.collectionRef = collection;
			//Activate spinner
			$M.toggleWait(true);
			// Refresh collection
			collection.refreshRecords().then(function() {
					$M.toggleWait(false);
			});
			this.dataSet = this.dataHost.formListRecordData;

		}
});
