/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2018
 */

Polymer({
	is: 'maximo-inventory-storeroom-flyout',
	behaviors: [BaseComponent, ItemMaster],
	listeners: {
        'maximo-lookup-selected' : '_returnFromLookup',
        'show-storeroom-allsites-flyout': '_showStoreroomForAllSites',
        'maximo-checkbox-changed':'_enabledRecording',
        'maximo-sub-component-changed':'_selectValueChange'
  	},
	properties : {		
    	record: {
    		type:Object,
  			notify:true,
  			value: {}
		},
		collection : {
			type : Object,
			notify:true
		},
		schema : {
			type : Object
		},
		storeroomData: {
			type: Object,
			notify: true,
			observer: '_setDefaultData'
		},
		 /**
		  * Cost Type filter
		  */
		costTypeDomainFilter: {
			type: Array,
		    value: function () {
		    	return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'COSTTYPE', 'selected': true} ]}];
		    } 	
	    },
	   /**
	    * Cost Type result
	    */
	    costTypeDomainSet: {
		  	type: Array,
		  	value: [],
		  	observer: '_costTypeList'
	    },
	    inventoryrecord: {
    		type:Object,
  			notify:true,
  			value: {}
		},
		addItem:{
			type:Boolean,
			notify: true,
			observer: '_setDefaultValue'
		},
		addItemDesc:{
			type:Boolean,
			value:false
		},
		
		additionalSetParam :{
   			type : String,
   			value: function () {
   				var itemsetDefaults = JSON.parse(sessionStorage.getItem('itemsetdefaults'));
   				var itemset='';
   				if(itemsetDefaults.defaultitemset){
   					itemset = itemsetDefaults.defaultitemset;
   				}
   				var additionalparams=[];
   				additionalparams.push('internalvalues=1&&oslc.where=itemsetid=%22'+itemset+'%22');
   				return additionalparams;
   			}
   		},
   		
   		changecosttype: {
   			type:Boolean,
   			value:false
   		},
   		
   		inventory_issueunit:{
			type:String,
		},
		additionalParam :{
			type:String
		}
	},
	
	ready: function() {
		this.record = {};
		this.$.costtypecollection.refreshRecords();		
		if(this.$.stdcost) {
			this.$.stdcost.label = $M.localize('uitext', 'mxapiitem', 'stdcost', [this.getCurrencyCode()]);
			this.$.stdcosttext.label = $M.localize('uitext', 'mxapiitem', 'stdcost', [this.getCurrencyCode()]);
		}
		if(this.$.avgcost) {
			this.$.avgcost.label = $M.localize('uitext', 'mxapiitem', 'avgcost', [this.getCurrencyCode()]);
			this.$.avgcosttext.label = $M.localize('uitext', 'mxapiitem', 'avgcost', [this.getCurrencyCode()]);
		}
		
		var lookup={"name":"itemunitmeasure","href":"/oslc/os/mxapiinventory/zombie","attributeMap":"record.issueunit:measureunitid"};
		 this.inventory_issueunit=Polymer.Base.create('maximo-text',{'id':'inventory_issueunit','class':'inventissue','lookup':lookup,'required':true,'maxlength':150});
		 this.inventory_issueunit.setAttribute('style', 'border:none; text-overflow: ellipsis;overflow: hidden;white-space: nowrap;flex: 0.7;');
		 this.inventory_issueunit.setAttribute('data-scope-id','inventory_issueunit');
		 this.inventory_issueunit.dataHost=this;
		 $j(this.inventory_issueunit.$.input).css('border-bottom','none');	
		 $j(this.inventory_issueunit.$.input)[0].setAttribute('readonly',true);
		 
		 if(this.record && this.record.issueunit){
			 this.inventory_issueunit.value=this.record.issueunit;
		 }
		 else{
			 this.inventory_issueunit.placeholder=$M.localize('uitext', 'mxapiitem', 'enterissueunit');	 
		 }
		 
		 this.$.condition_lookup.setAttribute('hidden',true);
		 this.$.condition_field.setAttribute('hidden',true);
		 $j(this.$.condition_field).addClass('adjust-condition_fieldnew');
	},

	_setDefaultData: function(){
		$j(this).find('maximo-text.lookuponly input').attr('readonly', true);
		$j(this).find('#editlotid').attr('required', false);
		$j(this).find('#editlotid_wrapper').css('display', 'flex');
		var inventorycost={};
				
		if(this.storeroomData.record){
			this.record = this.storeroomData.record;			
			this.issueunit = this.record.issueunit;
			if(this.issueunit){
				this.inventory_issueunit.value=this.issueunit;
			}
			else{
				this.inventory_issueunit.value='';
			}
			this.status = this.record.status;
			this.addtostoresiteid = this.record.siteid;
			this.itemnum=this.record.itemnum;
			
			$j(this.$.editbinid).find('input').attr('readonly', true);
			$j(this.$.editbinid).removeAttr('auto-expand');
			$j(this.$.editlotid).find('input').attr('readonly', true);
			$j(this.$.editlotid).removeAttr('auto-expand');
			
			if(!((this.record.costtype_maxvalue === 'LIFO') || (this.record.costtype_maxvalue === 'FIFO'))) {
				 inventorycost=this.record.invcost;
			 }
			else{
				 inventorycost=this.record.invlifofifocost;
			}	
			if((this.record.item && (this.record.item.conditionenabled || this.record.item.conditionenabled_localized ==='Y')) || 
					(this.record.conditionenabled === true || this.record.conditionenabled_localized ==='Y')){	
				if(inventorycost===undefined || $j.isEmptyObject(inventorycost)){
					this._sort(this.record.itemcondition,'condrate');
					this.$.conditioncode.value = this.record.itemcondition[0].conditioncode + ' (' + this.record.itemcondition[0].condrate +')';
					$j(this).find('#condition_field').removeAttr('hidden');
					$j(this).find('#condition_lookup').attr('hidden',true);
					$j(this).find('#condition_field_div').removeAttr('hidden');
				}else if(this.record.itemcondition && this.record.itemcondition.length <2){
					if(inventorycost && inventorycost.length){
						this._sort(inventorycost,'condrate');
						this.$.conditioncode.value=inventorycost[0].conditioncode + ' (' + inventorycost[0].condrate + ')';
					}
					$j(this).find('#condition_field').removeAttr('hidden');
					$j(this).find('#condition_lookup').attr('hidden',true);
					$j(this).find('#condition_field_div').removeAttr('hidden');
				}else if(this.storeroomData.action && this.storeroomData.action.identifier==='editaction' &&
						(this.record.conditionenabled === true || this.record.conditionenabled_localized==='Y')){
					this.$.conditioncode.value=this.record.conditioncode;
					$j(this).find('#condition_field').removeAttr('hidden');
					$j(this).find('#condition_lookup').attr('hidden',true);
					$j(this).find('#condition_field_div').removeAttr('hidden');
				}
				else{
					$j(this).find('#condition_lookup').removeAttr('hidden');
					$j(this).find('#condition_field').attr('hidden',true);
					$j(this).find('#condition_field_div').attr('hidden',true);
					this.additionalParam=this.additionalSetParam[0] +"and itemnum=%22"+this.record.itemnum+"%22";
					this.$.conditioncode.value='';		
				}
			} 
			else{
				$j(this).find('#condition_lookup').attr('hidden',true);
				$j(this).find('#condition_field').attr('hidden',true);
			}
			
			if(this.record.rotating ==='Y' || this.record.rotating === true) {
				this.$.balance_qty.setAttribute('hidden', true);
				this.$.balanceqtytext.removeAttribute('hidden');
				$j(this.$.balance_qty).find('input').attr('readonly', true);
			} else {
				if(this._isSigOptionGranted('MXAPIINVENTORY','CURBALADJ')) {
					this.$.balance_qty.removeAttribute('hidden'); 
					$j(this.$.balance_qty).find('input').attr('readonly', false);
					this.$.balanceqtytext.setAttribute('hidden', true);
				} else {
					this.$.balance_qty.setAttribute('hidden', true);
					this.$.balanceqtytext.removeAttribute('hidden');
					$j(this.$.balance_qty).find('input').attr('readonly', true);
				}
			}
			
			if(this.record.reorder){
				this.$.enabledrecording.checked=true;
			}else{
				this.$.enabledrecording.checked=false;
				if(this.isAuthorized('MXAPIINVENTORY:READ')){
					$j(this).find('.economic-record-field').css('display', 'none');
				}
			}
			this.$.editbinid.removeAttribute('hidden');
			this.$.editlotid.removeAttribute('hidden');
			this.$.balance_qty.removeAttribute('hidden');
			this.$.balanceqtytext.setAttribute('hidden', true);
			
			if(this.record.rotating ==='Y' || this.record.rotating === true){
				this.$.balance_qty.setAttribute('hidden', true);
				this.$.balanceqtytext.removeAttribute('hidden');
				$j(this.$.balance_qty).find('input').attr('readonly', true);
			}
			
			if(!this.storeroomData.action) {
				$j(this.$.editbinid).find('input').attr('readonly', false);
				$j(this.$.editbinid).attr('auto-expand', true);
				this.$.inventory_status.setAttribute('readonly', true);
				$j('.inventory_issueunit input').attr('readonly','true');
				this.$.enabledrecording.checked=false;
				this.$.economicRecordPoint.value='0.0';
				this.$.economicRecordQuentity.value='1.0';
				this.$.economicRecordLeadtime.value='0.0';
				this.$.stdcost.value= 0.0;
			    this.$.avgcost.value= 0.0;
			    this.$.count_frequency.value= 0.0;
				if(this.record.lottype_maxvalue && this.record.lottype_maxvalue === 'LOT'){
					$j(this.$.editlotid).find('input').attr('readonly', false);
					$j(this.$.editlotid).attr('auto-expand', true);
					this.$.editlotid.value = '';
					$j(this.$.editlotid).attr('placeholder', $M.localize('uitext', 'mxapiitem', 'enterlot'));
				} else {
					$j(this.$.editlotid).attr('placeholder', $M.getNoDataString());
				}
				
				$j(this.$.editbinid).attr('placeholder', $M.localize('uitext', 'mxapiitem', 'enterbin'));
				if(this.record.inventoryExists){
					this.$.inventory_location.setAttribute('readonly', true);
					$j(this).find('.inventory_issueunit').attr('readonly', true);
					$j(this.$.edit_storeroom_input_row2).attr('hidden', true);
					$j(this).find('.edit_economic_input_row2').attr('hidden', true);
				} else {
					$j(this.$.edit_storeroom_input_row2).removeAttr('hidden');
					$j(this).find('.edit_economic_input_row2').removeAttr('hidden');
				}
				this.changecosttype = true;
			} else if(this.storeroomData.action){
				$j(this.$.save_inventory).attr({'disabled':'true'});
				this.addItemDesc=true;
				this.$.inventory_location.setAttribute('readonly', true);
              	if(this.record.haschildinvbalance && (!this.record._hierarchylevel_ || this.record._hierarchylevel_ === 0)){
                    this.$.editbinid.setAttribute('hidden', true);
                    this.$.editlotid.setAttribute('hidden', true);
                    this.$.balance_qty.setAttribute('hidden', true);
                    this.$.balanceqtytext.removeAttribute('hidden');
              		if((this.record.item && (this.record.item.conditionenabled || this.record.item.conditionenabled_localized ==='Y')) || 
              				(this.record.conditionenabled === true || this.record.conditionenabled_localized==='Y')){
              			$j(this).find('#stdcostdiv, #avgcostdiv').hide();
              			$j(this).find('#costtype').css('flex', '0.15');
              			$j(this).find('#costtype').addClass('editfield-row2');
              			$j(this).find('#countFrequency').addClass('ccf');
              			$j(this).find('#statusunit-new').addClass('status-column');
              		}
              		else{
              			$j(this).find('#stdcostdiv, #avgcostdiv').show();
              			this.$.stdcost.value= this.record.invcost?this.record.invcost[0].stdcost:'';
              			this.$.avgcost.value= this.record.invcost?this.record.invcost[0].avgcost:'';
              		}
                    $j(this.$.balance_qty).find('input').attr('readonly', true);
              	} else if(this._isSigOptionGranted('MXAPIINVENTORY','CURBALADJ')) {
                	if((this.record.rotating ==='N' || this.record.rotating === false)) {
	                	this.$.balance_qty.removeAttribute('hidden');
            			$j(this.$.balance_qty).find('input').attr('readonly', false);
    					this.$.balanceqtytext.setAttribute('hidden', true);
            		} else {
	                	this.$.balance_qty.setAttribute('hidden', true);
            			$j(this.$.balance_qty).find('input').attr('readonly', true);
            			this.$.balanceqtytext.removeAttribute('hidden');
                	}
                 	$j(this).find('#stdcostdiv','#avgcostdiv').show();
                	this.$.stdcost.value= this.record.invcost?this.record.invcost[0].stdcost:'';
       				this.$.avgcost.value= this.record.invcost?this.record.invcost[0].avgcost:'';
                }	
                if(this.record.lottype_maxvalue && ((this.record.lottype_maxvalue === 'LOT'
                    && this.$.editlotid.value && this.$.editlotid.value.trim() !=='') 
                    || (this.record.lottype_maxvalue === 'NOLOT'))){
                    $j(this.$.editlotid).find('input').attr('readonly', true);
                    this.$.editlotid.placeholder = $M.getNoDataString();
                } else {
                    $j(this.$.editlotid).find('input').attr('readonly', false);
                    $j(this.$.editlotid).attr('placeholder', $M.localize('uitext', 'mxapiitem', 'enterlot'));
                }
                this.$.editbinid.placeholder = $M.getNoDataString();
                if(this._isSigOptionGranted('MXAPIITEM', 'COSTTYPE') === 1){
                	this.changecosttype = false;
                } else {
                	this.changecosttype = true;
                }
                this.$.count_frequency.value= this.record.ccf ? this.record.ccf : 0.0;
                var costTypeValues = this.$.costtypeselect.value.split('||');
                var costTypeMaxValue = costTypeValues.length ? costTypeValues[0] : '';
                this.async(function() {
	                if(costTypeMaxValue === 'FIFO' || costTypeMaxValue === 'LIFO') {
	                	this.$.balance_qty.setAttribute('hidden', true);
	            		$j(this.$.balance_qty).find('input').attr('readonly', true);
	    				this.$.balanceqtytext.removeAttribute('hidden');
	    				$j(this).find('#balanceqtytext_input').removeAttr('hidden');
	    				this.$.balance_qty.value = this.record.curbaltotal ? this.record.curbaltotal : 0.0;
					}
                }, 100);
            }
		}
	},
	
	/**
  	 * Refresh Collection
  	 */
	_handleCostTypeDomainSet : function(e){
  		if (this.costTypeData.length > 0) {
  			this.costTypeDomainSet = this.costTypeData[0].synonymdomain;
  		}
  	},
  	/**
  	 * Collection Refresh error
  	 */
  	_handleCostTypeDomainSetError : function(e){
  		this.costTypeDomainSet = [];
  	},
  	
  	/**
  	 * Parse cost type to create internalvalue string array for Drop Down
  	 */
  	_costTypeList : function(costTypeSet, record){
		  var costTypes = costTypeSet;
		  var externalValues = [];
		  if (costTypes.length > 0) {
			costTypes.forEach(function(member) {
				if (externalValues.indexOf(member.maxvalue) === -1 && member.defaults) {
					if(member.maxvalue !== 'ASSET' || (record.rotating === 'Y' || record.rotating === true)) {
						externalValues.push(member.maxvalue+'||'+member.value+':'+member.value);	
					}
				}
			});
		  }
		  this.$.costtypeselect.values=externalValues.toString();
  	},
	
	/**
     * Close dialog 
     */
  	_flyoutClose: function(){
  		if(!this.addItem){
  		   $M.getGlobalResource('inventorycollection').refreshRecords();
  		}
  		   this.owner.close();
	},
	
	/**
     * Enable/disable save button
     */
	_enableNext : function(){
		if(this.record && this.record.lottype_maxvalue &&
				this.record.lottype_maxvalue === 'LOT'){
			$j(this).find('#editlotid').attr('required', true);
			if(this.$.editlotid.value && this.$.editlotid.value.trim().length === 0) {
				this.lotcnt = '';
			}
		}		
		if(this.$.inventory_status.value && this.$.inventory_status.value && this.$.inventory_status.value.trim().length === 0) {
			this.status = '';
		}
		if(this.inventory_issueunit && this.inventory_issueunit.value && this.inventory_issueunit.value && this.inventory_issueunit.value.trim().length === 0) {
			this.issueunit = '';
		}	
		
		if(this.record && this.record.location !== '' && this.status && this.$.inventory_status.value && this.$.inventory_status.value.trim() !=='' &&
		   this.record.issueunit && this.inventory_issueunit.value && this.inventory_issueunit.value.trim() !=='') {
			if((this.record.item && (this.record.item.conditionenabled || this.record.item.conditionenabled_localized ==='Y')) || 
			   (this.record.conditionenabled === true || this.record.conditionenabled_localized==='Y')){
					if(this.$.conditioncodelookUp.value.trim() !=='' || this.$.conditioncode.value.trim() !='' || this.record.haschildinvbalance){
						$j(this.$.save_inventory).removeAttr('disabled');
					}
					else{
						$j(this.$.save_inventory).attr({'disabled':'true'});
						return;
					}	
			}
			if(this.record.lottype_maxvalue && this.record.lottype_maxvalue === 'LOT' 
				&& ((this.$.editlotid.value && this.$.editlotid.value.trim() !=='')
				|| (this.storeroomData.action && this.record.haschildinvbalance))) {
				$j(this.$.save_inventory).removeAttr('disabled');
			} else if(this.record.lottype_maxvalue && this.record.lottype_maxvalue === 'NOLOT'){
				$j(this.$.save_inventory).removeAttr('disabled');
			} else {
				$j(this.$.save_inventory).attr({'disabled':'true'});
			}
		}
		else{
			$j(this.$.save_inventory).attr({'disabled':'true'});
		}
	},
	/**
     * Set value of state and country on select from lookup
     */
	_returnFromLookup: function(e){		
		if(e.target.id ==='inventory_status' ){
			this.status = e.detail.records[0].value;
			this.$.inventory_status.value= e.detail.records[0].description;
		} else if(e.target.id ==='inventory_issueunit' ){
			this.issueunit = e.detail.records[0].measureunitid;
			this.inventory_issueunit.value = e.detail.records[0].measureunitid;
        } else if(e.target.id ==='inventory_location' ) {
        	if(!this.storeroomData.action){
        		$j('maximo-inventory-additemtostore')[0].fire('maximo-lookup-selected', e.detail);
        	}        	
        } else if(e.target.id==='inventory_item'){
        	this.addItemDesc=true;
        	this.itemnum = e.detail.records[0].itemnum;
        	this.$.inventory_desc.value=e.detail.records[0].description;
        	$j('maximo-storeroom-detail')[0].fire('maximo-lookup-selected', e.detail);  
        } else if(e.target.id ==='dfltbinid'){
            this.$.dfltbinid.value = e.detail.records[0].binnum;
        }
        else if(e.target.id==='conditioncodelookUp'){
        	this.$.conditioncodelookUp.value = e.detail.records[0].conditioncode + ' (' + e.detail.records[0].condrate + ')';
        }
		this._enableNext();		
		$j(e.target).find('input').focus();
	},
	
	/**
     * Create/Update inventory
     */
	_addItemToStoreroom: function(e){
		var inventoryData = {};
		var invcosts = [];
		var invcost = {};
		inventoryData = this.record;
		
		var conditioncoderate= this.$.conditioncode.value.replace(/[{()}]/g, '');
		
		inventoryData.binnum = this.$.editbinid.value? this.$.editbinid.value.trim() : '';
		inventoryData.lotnum = this.$.editlotid.value? this.$.editlotid.value.trim() : '';
		inventoryData.curbal = this.$.balance_qty.value? this.$.balance_qty.value : '0';
		inventoryData.siteid = this.addtostoresiteid;
		inventoryData.dfltbin = this.$.dfltbinid.value? this.$.dfltbinid.value : '';
		inventoryData.deliverytime=this.$.economicRecordLeadtime.value;
		inventoryData.orderqty=this.$.economicRecordQuentity.value;
		inventoryData.minlevel=this.$.economicRecordPoint.value;
		inventoryData.orderunit=this.$.orderunit.value;
		inventoryData.ccf=this.$.count_frequency.value;
		if(this.$.enabledrecording.checked===true){
			inventoryData.reorder=1;
		}
		else{
			inventoryData.reorder=0;
		}
		var costTypeValues = this.$.costtypeselect.value.split('||');
		var costTypeMaxValue = costTypeValues.length ? costTypeValues[0] : '';
		var costTypeValue = (costTypeValues.length > 1) ? costTypeValues[1] : '';
		
		inventoryData.costtype = costTypeValue;
		inventoryData.costtype_maxvalue = costTypeMaxValue;
		if(this.record.invcost && this.record.invcost.length) {
			this._sort(this.record.invcost,'condrate');
			invcost = this.record.invcost[0];
		}
		if(!((costTypeMaxValue === 'LIFO') || (costTypeMaxValue === 'FIFO'))) {
			invcost.stdcost = this.$.stdcost.value? this.$.stdcost.value : '0';
			invcost.avgcost = this.$.avgcost.value? this.$.avgcost.value : '0';
			invcost.conditioncode = conditioncoderate.split(' ')[0]==='' ? invcost.conditioncode : conditioncoderate.split(' ')[0];
			invcost.condrate = conditioncoderate.split(' ')[1]===undefined ? invcost.condrate : conditioncoderate.split(' ')[1];
			invcosts.push(invcost);
		}

		inventoryData.invcost = invcosts;
		
		if(this.status){
			inventoryData.status = this.status;
		}
		if(this.issueunit){
			inventoryData.issueunit =this.issueunit;
		}
		var additemtostoreroom=$j('maximo-inventory-additemtostore')[0];
		if(!additemtostoreroom){
			additemtostoreroom = Polymer.Base.create('maximo-inventory-additemtostore',{'id':'additemtostoreroom','name':'additemtostoreroom', 'editable':true});
		}		  
		if(this.record.inventoryExists){
			additemtostoreroom._createInvBalance(e, {'record' : inventoryData},this.addItem);
		} else if(this.storeroomData.action && this.storeroomData.action.identifier === 'editaction'){
			additemtostoreroom._updateInventory(e, {'record' : inventoryData},this.addItem);
		} else {
			additemtostoreroom._createInventory(e, {'record' : inventoryData},this.addItem);
		}
		
		this._flyoutClose();
	 },
	 
	 _showStoreroomForAllSites: function(e){
		 $j('maximo-inventory-additemtostore')[0].fire('show-storeroom-allsites');
	 },
	 /**
	  * Show Hide Div based on EnableReordering Checkbox
	  */
	 _enabledRecording: function(){
		 if(this.$.enabledrecording.checked){
			 $j(this).find('.economic-record-field').css('display', 'flex');
		 }
		 else{
			 if(this.isAuthorized('MXAPIINVENTORY:READ')){
				$j(this).find('.economic-record-field').css('display', 'none');
			 }
		 }
	 },
	 
	 _setDefaultValue:function(){
		 if(this.addItem){
			 this._enabledRecording();
			 this.$.conversion_field.style.display='block';
			 this.$.selected_storeroom.style.display='none';
			 $j(this).find('#balanceqtytext_input').attr('hidden', true);
			 this.$.dfltbin.style.display='none';
			 $j(this.$.conversion_div).addClass('adjust-conversion-div');
			 $j(this).find('#binedit-field').addClass('adjust-bineditfield');
			 $j(this).find('#lotedit-field').addClass('adjust-loteditfield');
			 $j(this.$.conversion_field_new).addClass('adjust-conversion-fieldnew');
			 $j(this.$.conversion_field).addClass('adjust-conversion-field');
			
			 if(!this.record || (this.record && (this.record.rotating ==='N' || !this.record.rotating))){
				 if(this.record && this.record.haschildinvbalance && (!this.record._hierarchylevel_ || this.record._hierarchylevel_ === 0)){
						if(this.record.item && (this.record.item.conditionenabled || this.record.item.conditionenabled_localized ==='Y')){
	              			$j(this).find('#stdcostdiv, #avgcostdiv').hide();
	              		}
	              		else{
	              			$j(this).find('#stdcostdiv, #avgcostdiv').show();
	              			this.$.stdcost.value= this.record.invcost?this.record.invcost[0].stdcost:'';
	              			this.$.avgcost.value= this.record.invcost?this.record.invcost[0].avgcost:'';
	              		}	 
				 
					 this.$.balance_qty.setAttribute('hidden', true);
					 this.$.editbinid.setAttribute('hidden', true);
					 this.$.editlotid.setAttribute('hidden', true);
					 $j(this).find('#balanceqtytext_input').removeAttr('hidden');
					 $j(this.$.balanceQty).addClass('adjust-balanceQty');
	              	 $j(this.$.balance_qty).find('input').attr('readonly', true);
				 }
			 else{
				if(this._isSigOptionGranted('MXAPIINVENTORY','CURBALADJ')) {
					$j(this).find('#balanceqtytext_input').attr('hidden', true);
					$j(this).find('#balance_qty_wrapperComponent').css('display', 'flex');
				 	$j(this).find('#balance_qty_label').css('padding', '5px 10px 0px 0px');
				 	$j(this.$.balanceQty).addClass('adjust-balanceQty');
				 	if(!this.storeroomData){
					 	this.$.economicRecordPoint.value='0';
					    this.$.economicRecordQuentity.value='1';
					    this.$.economicRecordLeadtime.value='0';
					    this.$.balance_qty.value='0';
					    this.$.stdcost.value= 0;
					    this.$.avgcost.value= 0;
					    this.$.count_frequency.value= 0;
					 	}
				 	else{
				 		 $j(this).find('#stdcostdiv,#avgcostdiv').show();
				 		 this.$.stdcost.value= this.record.invcost && this.record.invcost.length ? this.record.invcost[0].stdcost : undefined;
						 this.$.avgcost.value= this.record.invcost && this.record.invcost.length ? this.record.invcost[0].avgcost : undefined;
				 	}
				} else {
					this.$.balance_qty.setAttribute('hidden', true);
					$j(this).find('#balanceqtytext_input').removeAttr('hidden');
					$j(this.$.balanceQty).addClass('adjust-balanceQty');
		            $j(this.$.balance_qty).find('input').attr('readonly', true);
				}
			  }
			} else if(this.record && (this.record.rotating ==='Y' || this.record.rotating)){
				  this.$.editbinid.setAttribute('hidden', true);
				  this.$.editlotid.setAttribute('hidden', true);
				  this.$.balance_qty.setAttribute('hidden', true);
	              $j(this).find('#balanceqtytext_input').removeAttr('hidden');
	              $j(this.$.balanceQty).addClass('adjust-balanceQty');
	              $j(this.$.balance_qty).find('input').attr('readonly', true);
			 }
			 
			this.$.balance_qty.label = $M.localize('uitext', 'mxapiinventory', 'Startingbalance');
			 if(this.addItemDesc){
				 this.$.balance_qty.label = $M.localize('uitext', 'mxapistoreloc', 'newbalance');
				 this.$.inventory_desc.value=this.storeroomData.record.itemdescription;
				 this.$.dfltbin.style.display='block';
				 }
			 this.inventory_issueunit.label=$M.localize('uitext', 'mxapiitem', 'issueunit');
			 $j(this).find('#edit_storeroom_input_row2')[0].append(this.inventory_issueunit);
			 $j(this).find('#edit-row1').addClass('edit-row1-border');
			 
			 $j(this).find('#costtype').addClass('edit-field-old');
     		 $j(this).find('#stdcostdiv').addClass('edit-field-old');
     		 $j(this).find('#avgcostdiv').addClass('edit-field-old');
			 
		 }else{
			 this.$.conversion_field.style.display='none';
			 this.$.inventory_location.style.display='block';
			 this.$.balanceQty.style.display='block';		 
			 $j(this.$.conversion_div).addClass('adjust-conversion-divfirst');
			 $j(this).find('#issue-unit_div')[0].append(this.inventory_issueunit);
		 }
		 this.addItemDesc=false;
		 $j(this).find('#inventory_item_input')[0].setAttribute('readonly', true);
	 },
	 
	 /**
	  * _showInfo will show the Tooltip.
	 */
	_showInfo: function(e) {
		e.stopPropagation();
		var elementId = e.target.closest('maximo-button').dataset.scopeId+'_label';
		var tooltip = this.$[elementId];
		$M.showTooltip(tooltip, e.currentTarget);
	},
	/**
	 * Checks if mouse hovers for more than 350 milisecs
	 */
	_showInfoHover: function(e) {
		var icon = e.currentTarget;
		var self = this;
		if(icon.hoverTimer){
			window.clearTimeout(icon.hoverTimer);
		}
		
		$j(icon).one('mouseleave', function(){
			if(icon.hoverTimer){
				window.clearTimeout(icon.hoverTimer);
			}
			if(this.isAuthorized('MXAPIINVENTORY:READ')){
				$M.tooltip.hide();
			}
		});
	},
	
	_selectValueChange: function() {
		var costTypeValues = this.$.costtypeselect.value.split('||');
		var costTypeMaxValue =  costTypeValues.length ? costTypeValues[0] : '';
		if(this.storeroomData.action) {
			if((costTypeMaxValue === 'FIFO' || costTypeMaxValue === 'LIFO') ||
					(this.record.haschildinvbalance && (!this.record._hierarchylevel_ || this.record._hierarchylevel_ === 0)) || 
					(this.record.rotating)){
	        	this.$.balance_qty.setAttribute('hidden', true);
	    		$j(this.$.balance_qty).find('input').attr('readonly', true);
				$j(this).find('#balanceqtytext_wrapper').closest('maximo-text').removeAttr('hidden');
				$j(this).find('#balanceqtytext_input').removeAttr('hidden');
				this.$.balance_qty.value = this.record.curbaltotal ? this.record.curbaltotal : 0.0;
				this.$.balanceqtytext.value = this.record.curbaltotal ? this.record.curbaltotal : 0.0;
			}
			else {
				$j(this).find('#balance_qty_wrapperComponent').closest('maximo-numeric-spinner').removeAttr('hidden');
	    		$j(this.$.balance_qty).find('input').attr('readonly', false);
	    		this.$.balanceqtytext.setAttribute('hidden', true);
			}
			this._enableNext();
		}
		if(costTypeMaxValue === 'FIFO' || costTypeMaxValue === 'LIFO') {
			$j(this).find('#stdcost_wrapperComponent').closest('maximo-numeric-spinner').attr('hidden', true);
			$j(this).find('#avgcost_wrapperComponent').closest('maximo-numeric-spinner').attr('hidden', true);
			$j(this.$.stdcost).find('input').attr('readonly', true);
			$j(this.$.avgcost).find('input').attr('readonly', true);
			$j(this).find('#stdcosttext_wrapper').closest('maximo-text').removeAttr('hidden');
			$j(this).find('#avgcosttext_wrapper').closest('maximo-text').removeAttr('hidden');
			$j(this).find('#stdcosttext').removeClass('hide-std-avg-cost');
			$j(this).find('#avgcosttext').removeClass('hide-std-avg-cost');
		}
		else {
			$j(this).find('#stdcost_wrapperComponent').closest('maximo-numeric-spinner').removeAttr('hidden');
			$j(this).find('#avgcost_wrapperComponent').closest('maximo-numeric-spinner').removeAttr('hidden');
			$j(this.$.stdcost).find('input').attr('readonly', false);
			$j(this.$.avgcost).find('input').attr('readonly', false);
			$j(this).find('#stdcosttext_wrapper').closest('maximo-text').attr('hidden', true);
			$j(this).find('#avgcosttext_wrapper').closest('maximo-text').attr('hidden', true);
			$j(this).find('#stdcost').removeClass('hide-std-avg-cost');
			$j(this).find('#avgcost').removeClass('hide-std-avg-cost');			
		}
	},
	
	_getCostTypeValue: function(record){
		if(record && record.costtype && record.costtype_maxvalue){
			return record.costtype_maxvalue + '||' + record.costtype;
		}
		return '';
	},
	
	/**
	 * Sort a given list in descending order 
	 */
	_sort : function(list,attribute){
		if(list && list.length>0){
			list.sort(function(a,b){
			  return b[attribute] - a[attribute];
		  	});
		}
	}
	 
});