/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2018
 */
Polymer({
	is : 'maximo-inventory-update-flyout',
	behaviors : [ BaseComponent ],
	properties : {
		/**
		 * Set the Records in dialog
		 */
		storeroomData : {
			type : Object,
			notify : true,
			observer : '_setDefaultVal'
		},
		record : {
			type : Object,
			notify : true,
			value : {}
		},
		collection : {
			type : Object,
			notify : true
		},
		schema : {
			type : Object
		},
		flyoutWidth : {
			type : String,
			notify : true
		},
		flyoutTopOffset : {
			type : String,
			notify : true
		}
	},
	/**
	 * Apply validation on store field
	 */
	_storeFieldsValidate : function(e) {
		var saveButton = this.$.savebutton;
		var newBalance = this.$.invnewbalance.$.valueElement.value;
		newBalance = parseFloat(parseFloat(newBalance).toFixed(2));
		
		var stdcost=this.$.stdcost.value;
		var avgcost=this.$.avgcost.value;
		
		if (newBalance >= 0 || stdcost>=0 || avgcost>=0) {
			$j(saveButton).attr({'disabled' : false});
		} else {
			$j(saveButton).attr({'disabled' : true});
		}
	},
	_isNumber : function(e) {
		var evt = e;
		var key = evt.keyCode || evt.which;
		key = String.fromCharCode(key);
		var regex = /[-+]?[0-9]*[.,]?[0-9]+/;
		if (evt.keyCode !== 8 && evt.keyCode !== 46) {
			if (!regex.test(key)) {
				evt.returnValue = false;
				if (evt.preventDefault) {
					evt.preventDefault();
				}
			}
		}
	},
	/**
	 * Call validation method on every click
	 */
	attached : function(e) {
		var _self = this;
		$j(_self)
				.find('.newbalancespinner input, .newbalancespinner .divAjust, .invcost-row input, .invcost-row .divAjust')
				.on('keyup click', function(e) {
					_self._storeFieldsValidate(e);
				});
		
		$j(_self)
				.find('.newbalancespinner input, .newbalancespinner .divAjust, .invcost-row input, .invcost-row .divAjust')
				.on('keypress', function(e) {
					_self._isNumber(e);
				});
		
		$j(this.$.editinvflyout).css({
			'width' : this.flyoutWidth
		});

		if (window.navigator.userAgent.toLowerCase().indexOf('ipad') > -1) {
			var last_h = $j(window).innerHeight(); // store the initial height.

			var virtual_keyboardHeight = 423;
			if (/CriOS/.test(navigator.userAgent)) {
				virtual_keyboardHeight = 466;
			}
			$j(_self)
					.find('.newbalancespinner input')
					.on(
							'click',
							function(e) {
								if (window.innerHeight < window.innerWidth) {
									virtual_keyboardHeight = 517;
								}
								if (window.innerHeight < window.innerWidth && /CriOS/.test(navigator.userAgent)) {
									virtual_keyboardHeight = 560;
								}
								var flyOutHeight = $j(_self).closest('#flyout')
										.css('top');
								flyOutHeight = parseFloat(flyOutHeight);
								this.flyoutTopOffset = flyOutHeight;
								if ((last_h - flyOutHeight) < virtual_keyboardHeight) {
									var virtualOffset = last_h - virtual_keyboardHeight;
									$j(_self).closest('#flyout').css('top',
											virtualOffset + 'px');
								}
							});

			$j(_self).find('.newbalancespinner input').on(
					'blur',
					function() {
						if (this.flyoutTopOffset) {
							$j(_self).closest('#flyout').css('top',
									this.flyoutTopOffset + 'px');
						}
					});

			window.onresize = function() {
				$j(_self).find('.newbalancespinner input').blur();
				_self.owner.close();
			};
		}

		if(this.$.stdcost) {
			this.$.stdcost.label = $M.localize('uitext', 'mxapiitem', 'stdcost', [this.getCurrencyCode()]);
			this.$.stdcosttext.label = $M.localize('uitext', 'mxapiitem', 'stdcost', [this.getCurrencyCode()]);
		}
		if(this.$.avgcost) {
			this.$.avgcost.label = $M.localize('uitext', 'mxapiitem', 'avgcost', [this.getCurrencyCode()]);
			this.$.avgcosttext.label = $M.localize('uitext', 'mxapiitem', 'avgcost', [this.getCurrencyCode()]);
		}
		
	},
	/**
	 * Set default value in dialog
	 */
	_setDefaultVal : function() {
		var _self = this;
		if (this.storeroomData) {
			this.record = this.storeroomData.record;
			this._returnInvCost(this.record);
			if(!this.record.conditionenabled){
                $j(this).find('#stdcost_wrapperComponent').hide();
  				$j(this).find('#avgcost_wrapperComponent').hide();
  				$j(this).find('#stdcosttext_wrapper').hide();
  				$j(this).find('#avgcosttext_wrapper').hide();
  				$j(this).find('#stdcostdiv').hide();
				$j(this).find('#avgcostdiv').hide();
  			}
			else {
				if(this.record.costtype_maxvalue === 'LIFO' || this.record.costtype_maxvalue === 'FIFO') {
	  				$j(this).find('#stdcost_wrapperComponent').hide();
					$j(this).find('#avgcost_wrapperComponent').hide();
					$j(this.$.stdcost).find('input').attr('readonly', true);
					$j(this.$.avgcost).find('input').attr('readonly', true);
					$j(this).find('#stdcosttext_wrapper').show();
					$j(this).find('#avgcosttext_wrapper').show();
					$j(this).find('#stdcosttext').removeClass('hide-std-avg-cost');
					$j(this).find('#avgcosttext').removeClass('hide-std-avg-cost');
	  				this.$.invnewbalance_text.value = this.record.curbaltotal ? this.record.curbaltotal : '0.0';
	  			}
	  			else {
	  				$j(this).find('#stdcostdiv').show();
	  				$j(this).find('#avgcostdiv').show();
	  				$j(this).find('#stdcost_wrapperComponent').show();
	  				$j(this).find('#avgcost_wrapperComponent').show();
	  				$j(this.$.stdcost).find('input').attr('readonly', false);
	  				$j(this.$.avgcost).find('input').attr('readonly', false);
	  				$j(this).find('#stdcosttext_wrapper').hide();
	  				$j(this).find('#avgcosttext_wrapper').hide();
	  				$j(this).find('#stdcost').removeClass('hide-std-avg-cost');
	  				$j(this).find('#avgcost').removeClass('hide-std-avg-cost');	
	  				
	  			}
			}
			
		}
		this.async(function() {
			$j(_self).find('.newbalancespinner input').focus();
		}, 300);
	},
	/**
	 * Close dialog
	 */
	_flyoutClose : function() {
		this.owner.close();
	},
	_adjustBalance : function(invData) {
		var self = this;
		this.$.invbalancesresource.updateAction('wsmethod:adjustCurrentBalance', invData, 'itemnum').then(
			function() {
				$M.getGlobalResource('inventorycollection').refreshRecords().then(function() {
					$M.toggleWait(false);
					self.owner.close();
				});

			}, function(error) {
				$M.showResponseError(error);
				$M.toggleWait(false);
				self.owner.close();
			});
	},
	
	_updateCurrentBalance : function(e) {
		var self = this;
		$j(self).find('.newbalancespinner input').blur();
		$M.toggleWait(true);
		var invBalData = {};
		var newInvBalance = 0;
	    if(this.$.invnewbalance.value !== 0) {
	       	newInvBalance = this.$.invnewbalance.value;
	    }
				
		var confirmResourceUri;
		confirmResourceUri = this.record.localref;		
		invBalData.newBalance = newInvBalance.toString();
		invBalData.conditioncode = this.$.conditioncode.label;
		invBalData.stdcost=parseFloat(this.$.stdcost.value);
		invBalData.avgcost=parseFloat(this.$.avgcost.value);	
		
		confirmResourceUri += '?editmode=1';
		this.$.invbalancesresource.resourceUri = confirmResourceUri;
		this._adjustBalance(invBalData);
		
	},

	_recordUpdatedSuccessHandler : function(e) {
		$M.toggleWait(false);
		var targetResource = e.target;
		if (targetResource && targetResource.__data__ && targetResource.__data__.resourceData) {
			this.record = targetResource.__data__.resourceData;
		}
	},

	_recordUpdatedErrorHandler : function(e) {
		$M.toggleWait(false);
		if (this.targetelement) {
			this._setPreEditValues(this.targetelement);
		}
		var error = e.detail;
		if (error && error.request && error.request.response && error.request.response.Error) {
			$M.notify(error.request.response.Error.message, $M.alerts.error);
		}
	},
	
	 _returnInvCost: function(record){
		 var self=this;
		 var invcost=record.invcost ? record.invcost : [];
		 invcost.forEach(function(currentRecord) {
				if (currentRecord.conditioncode == record.conditioncode) {
					self.$.stdcost.value = currentRecord.stdcost_localized;
					self.$.avgcost.value = currentRecord.avgcost_localized;
				}
			});
	 },
	 
	 /**
     * Enable/disable save button
     */
	_enableNext : function(){
		if(this.record.costtype_maxvalue === 'LIFO' || this.record.costtype_maxvalue === 'FIFO') {
			$j(this.$.savebutton).attr({'disabled': true});
		} else if(this.$.invnewbalance && this.$.invnewbalance.value !== null && this.$.invnewbalance.value !== undefined) {
			$j(this.$.savebutton).attr({'disabled': false});
		} else {
			$j(this.$.savebutton).attr({'disabled': true});
		}
	},

});
