/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2019
 */
Polymer({
	is: 'maximo-inventory-create-picklist',	
  	behaviors: [BaseComponent],
  	listeners: {
  		'date-changed': 'onReqDateChanged'
  	},
    properties: {
    	
    	/**
    	 * Item record
    	 */
    	record : {
			type: Object,
			notify: true
		},
		collection : {
			type : Object
		},
		showitemset: {
			type: Boolean,
			value: false
		},
		schema : {
			type : Object
		},
		linecount : {
			type: Number,
			value: 0
		},
		storeroom : {
			type: String,
			value: ''
		},
		picklistDefaults: {
			type: Object,
			value: {}
		},
		selectedReservations: {
			type: Array,
			value: []
		},
		dateformat: {
			type: String,
			value: function(){
				if($M.isMobile()){
					return 'YYYY-MM-DD';
				} else {
					return $M.maxauth.whoami.dateformat.toUpperCase();
				}
			}
		}
	},
	
	/**
     * Display PickList Main page on cancel
     */
	_cancelCreatePickList: function(e){
		$M.setSubRoute('reservations');
		this.fire('display-itempage', {'pagename': 'reservation_list', 'tabName':'reservation', 'fromTabName':'createPickList'});
  	},
    	 
    /**
     * Enable/Disable Add Details button
     */
    _enableNext : function(dataset, selected){
		if(this.$.picklistDescription.value && this.$.picklistDescription.value.trim().length && this.$.listId.value && this.$.listId.value.trim().length && this.$.requiredDate.value 
			&& this.$.requiredDate.value.trim().length && this.$.storeroom.value && this.$.storeroom.value.trim().length){	
		    $j(this.$.gonextitem).removeAttr('disabled');
		}
		else {
			$j(this.$.gonextitem).attr({'disabled':'true'});
		}
	},
	
	setDefaultValues: function(){
		var dates=[];
		var dateformat = $M.maxauth.whoami.dateformat.toUpperCase();
		var selectedReservations = this.selectedReservations;
		
		if($M.isMobile()){
			dateformat = 'YYYY-MM-DD';
		} 
		this.$.listId.value = this.picklistDefaults.picklistnum;
		this.$.linecount.value = this.linecount;
		this.$.storeroom.value = this.storeroom;
		if(selectedReservations.length){
			selectedReservations.forEach(function(reservation, index){
				dates.push(new Date(selectedReservations[index].requireddate));
			});
		}		
		var minDate=new Date(Math.min.apply(null,dates));
		this.$.picklistDescription.value = '';
		this.$.requiredDate.value = moment(minDate).format(dateformat);
		this._enableNext();
	},
	
	_createPickList: function(e){
    	var plData = {};
    	plData.msr = {};
    	plData.msr.selection = [];
    	var plLines = this.selectedReservations;
    	var reqdate = this.$.requiredDate.value.trim();
    	
    	plData.plnum = this.$.listId.value.trim();
    	if(this.$.picklistDescription.value)
    		plData.pldescription = this.$.picklistDescription.value.trim();
    	if($M.maxauth.whoami.dateformat === "dd/MM/yy"){
    		plData.reqdate = moment(reqdate,'YYYY-MM-DDThh:mm:ss').format('YYYY-MM-DDThh:mm:ss');
    	}
    	else{
    		plData.reqdate = moment(reqdate).format('YYYY-MM-DDThh:mm:ss');
    	}
    	plData.storeroom = this.$.storeroom.value.trim();
    	plData.siteid = plLines[0].siteid;
    	plLines.forEach(function(plLine, index){
    		plData.msr.selection[index] = {};
    		plData.msr.selection[index].invreserveid = plLine.invreserveid;
    	});
    	
    	var self = this;
    	$M.toggleWait(true);
    	var picklistresource = this.$.picklistcreateresource;
    	var responseProperties = 'picklistnum,description,createddate,status,fromstoreloc,requireddate--reqdatepicklist';
        picklistresource.resourceUri = $M.maxauth.baseUri + '/oslc/service/inventory?responseos=MXAPIINVPICKLIST';
        picklistresource.updateAction('wsmethod:createPickList', plData, responseProperties).then(
        	function(result){
        		if(result.response) {
        			$M.toggleWait(false);
        			var plrecord = result.response;					
					$M.setSubRoute('reservations/pickList/details');
					self.fire('display-itempage', {'pagename': 'pickListDetail', 'tabName':'pickList', 'record': plrecord, 'schema': self.schema});
        		}
        	}, function(error){
	        	$M.toggleWait(false);
	        	$M.showResponseError(error);
        	});
	},
	
	onReqDateChanged: function(e){
		if(e.detail && e.detail.value === ''){
			this.$.requiredDate.value = moment().format(this.dateformat);
		}
		this._enableNext();
	}
	
});
