/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2019
 */
Polymer({
	is: 'maximo-submitsr-additionaldetails',
  	behaviors: [BaseComponent,ServiceRequestBehavior],
    properties: {
		label: {
			type: String,
			value: 'Panel'
		},
		
		icon: {
			type: String,
			value: 'title'
		},		
		title: {
			type: String,
			value: 'title'
		},
		
    	selectedRecord: {
            type: Object,
            notify: true,
    		observer:'_cleanAttributeList'
        },
        
		additionalParams :{
			type : Object
		},

        /**
    	 * Attribute Domain result
    	 */
        
		attrDomain: {
    		type: Object,
    		notify: true ,
    		observer:'_configureDomain'
    	},
    	
    	/**
    	 * Attribute Domain filter
    	 */
    	attribDomainFilter: {
    		type: Array,
    		value: function () {
    			return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'DATATYPE', 'selected': true} ]}];
    		} 	
    	},
    	
    	ALN_DOMAIN: {
			type: String 
		},
		
		EXT_DOMAIN: {
			type: Array,
			value:[]
		},
		
		DATE_DOMAIN: {
			type: String 
		},
		
		
		TABLE_DOMAIN: {
			type: String 
		},
		
		NUMERIC_DOMAIN: {
			type: String 
		},
		
		attributeList: {
			type: Array,
			value:[]
		},
    	 
	},
	listeners :{
		'maximo-lookup-selected' : '_returnFromLookup',
  		'maximo-typeahead-selected' : '_returnFromLookup',
  		'input' : '_inputChanged'
	},
	
	
	 /**
	 * refresh Items ids
	 */ 
	 _refreshIDs: function(){
		try {
			var self = this;
			this.async(function(){
				
				let components= ['maximo-field', 'maximo-button'];
				components.forEach(function(comp){
					let list =  $j( self.$.additionaldetailsWrapper).find(comp);
					for (let i=0 ; i<  list.length ; i++ ){
						let checkbox = list[i];
						checkbox._setIdentification();
					}
				});
			   
		   },500); 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	 },
	
	_configureDomain : function(){
		try {
			var self = this;
			var attrDomainList = this.attrDomain[0].synonymdomain;
			if(attrDomainList.length>0){
				attrDomainList.forEach(function(atdom){
					if(atdom.maxvalue=='ALN'){
						self.ALN_DOMAIN= atdom.value;
						self.EXT_DOMAIN.push(atdom.value);
					}else if(atdom.maxvalue=='NUMERIC'){
						self.NUMERIC_DOMAIN= atdom.value;
					}else if(atdom.maxvalue=='MAXTABLE'){
						self.TABLE_DOMAIN= atdom.value;
					} else if(atdom.maxvalue=='DATE'){
						self.DATE_DOMAIN= atdom.value;
					} 
					
					
					
				});
			}
			this._inputChanged();
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
		
	},
	
	_handleDataError : function(){
		
	},
	
	ready: function() {
	},
	
	attached : function(){
		
	},
	
	_goNext : function(e) {
		try {
			var submitinfo = {
				step: 'Issue',
			};

			submitinfo.value = {
					description : this._submitInfo.Issue.contents.description,
					templateid: this._submitInfo.Issue.contents.templateid,
					hasticketspec: true,
					ticketspec: this.selectedRecord.tktemplatespec
				};
	 
		this.fire('go-next',  submitinfo);
		} catch(err) {
			this._displayUnexpectedError(err);
		} 
	},
	
	initPage: function() {
		this.attributeList=[];
	},
	
    _cleanAttributeList : function(){
		try {
			this.attributeList=[];
			this._refreshIDs();
			this._initRTE();
			this._inputChanged();
		} catch(err) {
			this._displayUnexpectedError(err);
		}
	
	},
	
	
	/**
	 * check if all mandatory attributes have been filled
	 **/
	_inputChanged: function(e){
		try {
			var self = this;
			this.async(function(){
				var valid=true;
			
				if (self.$.tkspeclist &&  self.$.tkspeclist.collection  && self.$.tkspeclist.collection.userArray){
					self.$.tkspeclist.collection.userArray.forEach(function(attrib) {
						if (valid && attrib.mandatory ){
							
							if (attrib.assetattribute.domainid){
								if (self._isNotEmpty(attrib.numvalue) || self._isNotEmpty(attrib.alnvalue)){
									valid=true;
								} else {
									valid=false;
								}
							}else if (attrib.assetattribute.datatype === self.ALN_DOMAIN  ){
								if ( self._isNotEmpty(attrib.alnvalue)){
									valid=true;
								} else {
									valid=false;
								} 
							}else if (attrib.assetattribute.datatype === self.NUMERIC_DOMAIN  ){
								if ( self._isNotEmpty(attrib.numvalue)){
									valid=true;
								} else {
									valid=false;
								} 
							}else if (attrib.assetattribute.datatype === self.DATE_DOMAIN  ){
								if ( self._isNotEmpty(attrib.datevalue)){
									valid=true;
								} else {
									valid=false;
								} 
							}
						}
					})
				}
				self.$.additionaldetails.disableNext = !valid;
				return valid;
			}, 50);
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	_isNotEmpty:function(itemValue){
		try {
			if( itemValue !==undefined){
				if(typeof itemValue === 'string'){
					if( itemValue.trim()  !== '' ){
						return true
					}
				}else{
					return true
				}
				 
			 }  
			 return false;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	_disableNext: function(tktemplatespec) {
		try {
			var enable=true;
			if (tktemplatespec){
				var self = this;
				tktemplatespec.forEach(function(attrib) {
					if (attrib.mandatory && attrib.assetattribute.datatype === self.ALN_DOMAIN){
						enable = false;
					}	 
				})
			}
			return !enable; 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	_getDatatypeIntVal: function(localizedDatatype,attribDomainCollection){
		try {
			var attrDomain = this.attribDomainCollection[0].synonymdomain;
			var externalValues=[];
			var maxvalues=[];
				if(attrDomain.length>0){
					attrDomain.forEach(function(atdom){
						externalValues.push(atdom.value);
						maxvalues.push(atdom.maxvalue);
					});
				}
			var idx=externalValues.indexOf(localizedDatatype);
			return maxvalues[idx];
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
		
	},
	
	
	_configureDomainFields: function() {
		try {
			var self = this;
			this.attributeList.forEach(function(index){
				$j($j($j(self).find('#attr_'+index)[0]).find('input')[0]).attr('disabled', 'disabled');
			}); 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
		
		
	},

	_showDomain: function(attr , index) {
		try {
			if (attr.assetattribute.datatype ===this.TABLE_DOMAIN){
				return false;
			}
			
		   if (attr.assetattribute.domainid){
			   
			   if(this.attributeList.indexOf(index) < 0 ){
				   this.attributeList.push(index);
			   }
			 
			   this._initDynamicLookup  (attr.assetattribute.domainid, attr.assetattribute.datatype, attr.assetattribute.description);
				 return true;
		   }
			
			return false;	 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	_showAlnValue: function(attr, index) {
		try {
			if(this._showDomain(attr, index)){
				return false;
			}
			for(let i=0;i<this.EXT_DOMAIN.length;i++)
			{
				if (attr.assetattribute.datatype ===this.EXT_DOMAIN[i])
				{
				return true;
				}	
			}	
			
			return false; 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	_showDateValue: function(attr, index) {
		try {
			if (attr.assetattribute.datatype ==this.DATE_DOMAIN){
				return true;
			}	
			
			return false; 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	
	_showAlnDomain: function(attr, index) {
		try {
			if(this._showDomain(attr, index)){
				if (attr.assetattribute.datatype ==this.ALN_DOMAIN){
					return true;
				}
			}
			
			return false;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	_showNumValue: function(attr, index) {
		try {
			if(this._showDomain(attr, index)){
				return false;
			}
			if (attr.assetattribute.datatype ==this.NUMERIC_DOMAIN){
				return true;
			}	 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
	
	_showNumDomain: function(attr, index) {
		try {
			if(this._showDomain(attr, index)){
				if (attr.assetattribute.datatype == this.NUMERIC_DOMAIN){
					return true;
				}
			}
			return false;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},



   _isRequired: function(mandatory) {
		return mandatory;
	},

	
	_getAttrDesc: function(attr , attrunit) {
		try {
			var attrdesc = attr.assetattribute.description ? attr.assetattribute.description : attr.assetattrid; 
			var label = "";
			if (attrunit){
				label = attrdesc + "(" + attrunit + ")";
			} else {
				label = attrdesc;
			}
			return label;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},

	_attrHasDomain: function(domainid, datatype, description) {	
		try {
			if (domainid) {
				this._initDynamicLookup(domainid, datatype, description);
				return true;
			}
			else {
				return false;
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},

	_initDynamicLookup: function(domainid, datatype, description) {
		try {
			var lookup = {};
			lookup["objectName"] = "MXAPIDOMAIN";
			lookup["attributeNames"] = "description,value";
			lookup["displayAttributes"] = "description,value";
			lookup["label"] = "Select " + description;
			lookup["searchAttributeNames"] = "description,value";
			lookup["useSearchTerms"] = "true";
			lookup["layout"] = "sr-domain-card";
			$M._lookups[domainid] = lookup;
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},

	_setLookup: function(attr) {
		try {
			var domainid =attr.assetattribute.domainid;
			var maxdomainid = attr.assetattribute.maxdomain.maxdomainid
	        var  datatype = attr.assetattribute.datatype;
			var attributeMap;
			var index =  this.selectedRecord.tktemplatespec.indexOf(attr);
			var domainType = null;
			if (datatype === this.ALN_DOMAIN) {
				domainType = "alndomainvalue";
			//	attributeMap  = 'selectedRecord.tktemplatespec['+index+'].numvalue:description';
			}
			else if (datatype === this.NUMERIC_DOMAIN ) {
				domainType = "numdomainvalue";
			//	attributeMap  = 'selectedRecord.tktemplatespec['+index+'].alnvalue:description';
			}
			var whereClause = 'oslc.where=domainid=%22'+domainid+'%22';
			
			var result={};
			var lookup={};
			
		//	var lookup = {"name":" + domainid + '","href":"/oslc/os/mxapidomain/' + maxdomainid + '/' + domainType + '","additionalLookupParams":"' + whereClause + '"};
			lookup.name=domainid;  
			lookup.href='/oslc/os/mxapidomain/' + maxdomainid + '/' + domainType ;
			//lookup.attributeMap = attributeMap;
		  //  lookup.attribute='value';
		    lookup.placeholder= this.localize('uitext','mxapisr','EnterResponse');
		   
			lookup.additionalLookupParams = whereClause;
			result.lookup = lookup;
			result.placeholder= this.localize('uitext','mxapisr','EnterResponse');  
			
			result.readonly=true;
			result.readonlyLookup=true;
			result.autoExpand=true;
			 
		 
			return JSON.stringify(result);
		} catch(err) {
			this._displayUnexpectedError(err);
		}
			
	},
	
 
	
	_getPlaceholder: function( ){
		var result={};
		result.placeholder= this.localize('uitext','mxapisr','EnterResponse');
		return JSON.stringify(result);
	},

	renderPage: function(submitInfo, stackData) {
		try {
			this._submitInfo = submitInfo;
			this._stackData = stackData;
			var newStackData = [];
			if (stackData) {
				for (var idx = stackData.length -1; idx>=0; idx--) {
					if (stackData[idx].data && stackData[idx].data.length !== 0) {
						newStackData.push($M.cloneRecord(stackData[idx]));
					}
				}
			}
		
			this.$.additionaldetails.setStackList(newStackData);
			var self =this;
			this.async(function(){
				self._configureDomainFields();
			}, 400);
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
		
	},

	/**
	 * To set the values from Location lookup
	 */
    _returnFromLookup: function(e){
		try {
			let attribute;
			attribute = this._getAttribute(e.srcElement ? e.srcElement : e.target);
			
			if(e.detail.records[0] && this._isNotEmpty(e.detail.records[0].value) ) {
				if (attribute.assetattribute.datatype == this.NUMERIC_DOMAIN){
					attribute.numvalue = e.detail.records[0].value;
				}else{
					attribute.alnvalue = e.detail.records[0].value;
				} 
				
				if(e.detail.records[0].description){
					e.srcElement.value =e.detail.records[0].description;
					attribute.description = e.detail.records[0].description;
				}else{
					e.srcElement.value = e.detail.records[0].value;
					attribute.description = e.detail.records[0].value;
				}
			}else{
				e.srcElement.value = undefined;
				attribute.numvalue = undefined;
				attribute.alnvalue = undefined;
				attribute.description = undefined;
			}
		} catch(err) {
			this._displayUnexpectedError(err);
		}
    	
	},
	
	/**
	 * get the attribute associated with ui element
	 */
	_getAttribute: function(e){
		try {
			if("MAXIMO-FIELD" === e.tagName){
		    	return  e.record;
		    } 
		    return this._getAttribute(e.parentNode);
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		   
	   },
	
	
	/**
	 * get the instruction description
	 */
	_getInstructionDescription: function(selectedRecord){
		try {
			if(selectedRecord && selectedRecord.instractions_longdescription) {
				return selectedRecord.instractions_longdescription;
			}
			
			return  this.localize('uitext','mxapibase','NotEntered'); 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
	   
	},
	
	_hideInstruction: function(selectedRecord){
		try {
			if(selectedRecord && selectedRecord.instractions_longdescription) {
				return false;
			}
			
			return true; 
		} catch(err) {
			this._displayUnexpectedError(err);
		}
	   
	},
	
	
	_initRTE: function() {
		try {
			this.$.instruction.updateContent(this._getInstructionDescription(this.selectedRecord));
		} catch(err) {
			this._displayUnexpectedError(err);
		}
		
	},
	
});
