/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2017,2018
 */
Polymer({
    is: 'maximo-report-time-card',
    behaviors: [BaseComponent, DirtyBehavior],
    listeners: {
        'dirty-element': '_updateBorder',
        'maximo-lookup-selected': '_returnFromLookup',
        'maximo-reset-element':'_resetCard',
        'is-dirty-changed': '_updateBorder',
        'maximo-typeahead-selected' : '_returnFromLookup',
        'disable-time-list' : '_disablelist'
    },
    properties: {
        /**
         * the work order record
         */
        record: {
            type: Object,
            observer: '_clearDirty'
        },
        wostatus: {
            type: String
        },
        /**
         * Interval in seconds to 
         * undo delete operation
         */
        undoIntervalSecs: {
            Type: Number,
            value: 3,
            readOnly: true
        },
        /**
    	 * LT Type domain filter
    	 */
    	ltTypeSet: {
    		type: Array,
    		value: function () {
    			return [{'filtertype': 'SIMPLE', 'field': 'domainid', 'availablevalues': [ {'value': 'LTTYPE', 'selected': true} ]}];
    		} 	
    	},
    	
    	/**
		 * To identify new item card.
		 */
		newCard : {
			type : Boolean,
			value : false
		},
		
		/**
		 * To bypass update values.
		 */
		byPassUpdate : {
			type : Boolean,
			value : false
		},
		
		isDateDurationUpdated: {
			type : Boolean,
			value : false
		},
		/**
		* Array of Craft
		*/
		craftData : {
			type : Array
		},
		/**
         * To get craft clause.
         */
         craftClause: {
                type : String,
                computed: '_getCraftClause(laborcode)'
         },
         /**
         *  Laborcode Value.
         */
         laborcode: {
                type : String,
                value:''
         },
         /**
 		 * To get labor clause.
 		 */
 		 laborClause: {
 			type : String,
 			computed: '_getLaborClause(craft)'
 		 },
 		 craftcode: {
 			type : String,
 			value: ''	
 		 },
 		/**
  		 * To get premium clause.
  		 */
  		 premiumClause: {
  			type : String,
  			computed: '_getPremiumClause(craftcode)'	
  		 },
 		 premiumCodeURL: {
 			type : String,
 			value: ''	
 		 },
 		 premiumpay: {
 			type : String,
 			value: ''	
 		 }
    },
    attached: function() { 
    	this._getLaborClause(''); 
    	this.parentResetButton = this.$.divReset.id;
        this.showResetButton = true;
        this._toggleDeleteSaveButton();
        $j('.report_times_type').find('select').css('width', '90%');
        $j('.report_times_hours_label_spinner .custom-ellipsis-class').css('display','inline-block');
        this.async(function(){
    		$j('.report_time_premium_readonly .dynamiclabel .largelabel').css({'font-size':'22px', 'color':this.getComputedStyleValue('--Primary-gray70'), 'font-family':'MaximoBase-Light'});
    		if(!this.editable){
    			$j(this.$.report_time_person.$.inputDiv).removeAttr('required');
    			$j('.noDataField').find('[data-scope-id=childComponent]').css('margin-top', '8px');
    		}
        }, 350);
    },
    
    setTime : function(time){
    	var labTransTime = (time)? time : '';
	    if($M.isMobile() && labTransTime){
    		labTransTime = moment(labTransTime,$M.getMaxauth().whoami.timeformat).format('HH:mm');
    	} 
    	return labTransTime;
    },
    setDate : function(date){
    	var labTransDate = (date)? date : '';
    	if($M.isMobile() && labTransDate){
    		labTransDate = moment(labTransDate,$M.getMaxauth().whoami.dateformat.toUpperCase()).format('YYYY-MM-DD');
    	} 
    	return labTransDate;
    }, 

    /**
     * This method will toggle border when card is in edit mode
     */
    _updateBorder: function() {
	      this.setHighlight(this.isDirty);
	      this._toggleDeleteSaveButton();
	      this.fire('maximo-report-time-card-dirty');
    },
    /**
     * This method will toggle border when card is in edit mode
     */
    setHighlight: function(isHighlight) {
    	$j(this.$.report_time_innerDiv).toggleClass('dirtyBorder',isHighlight);
    	this._disablelist(isHighlight);
    },

    /**
     * Reset the values.
     */
    _clearDirty: function(e) {
        this.clearDirtyStatus();
        if(this.domHost && !this.isDirty){
        	this.domHost._disableHeader(false);
        }
    },
    /**
     * delete new cards.
     */
    _resetCard: function (e) { 
    	this.byPassUpdate = true;
    	var self = this;
    	this.async(function(){
    		self.byPassUpdate = false;
		}, 250);
    	
    	if (this.newCard) {
         	this._deleteTime(e);
        } 
    	else {
    		var temp = JSON.parse(JSON.stringify(this.record));
    		temp.finishdate_localized = null;
    		temp.finishtime_localized = null;
    		temp.regularhrs = null;
    		this.record = temp;
    	}
    	e.stopPropagation();
    },
	/**
     * Initiate delete planned time event.
     */
	_deleteTime: function(e) {
        e.stopPropagation();
        this.fire('disableAddTime', true);
        var self = this;       
        var performNewDeleteTime = function(){
        	self.isDirty = false;
        	self._disablelist(false);
        	if(self.domHost){
        		self.domHost._disableHeader(false);
        	}
        	self.remove();
        	if($j(self.domHost).find('maximo-report-time-card').length === 0){
        		self.domHost.$.nodatasection_labtranss.hidden = false;
            } 

        	return;
        };
       
        var target = e.target;
        var performDeleteTime = function(){
        	var record = self.domHost.$.reported_time_template.itemForElement(target);
        	var recordUri = (record && record.localref)?record.localref:'';
        	self.fire('deleteTimeCard', recordUri);
        	self.isDirty = false;
        	self._disablelist(false);
        	if(self.domHost){
        		self.domHost._disableHeader(false);
        	}
        };
        
        var revertDeleteTime = function(overlayNode){
        	if(!self.isDirty){
        		self.fire('disableAddTime', false);
            	//does nothing except remove the delete overlay
        	}
        };

        if (this.newCard){
        	 $M.createOverlay(this.$.report_time_card, this.localize('uitext','mxapiwodetail','reportitemdelete',[this.localize('uitext','mxapiwodetail','Time')]), performNewDeleteTime, revertDeleteTime, 
               		 this.undoIntervalSecs, this.parentElement.parentElement, null);
        }else{
        	$M.createOverlay(this.$.report_time_card, this.localize('uitext','mxapiwodetail','reportitemdelete',[this.localize('uitext','mxapiwodetail','Time')]), performDeleteTime, revertDeleteTime, 
              		 this.undoIntervalSecs, this.parentElement.parentElement, null);
        }
    },
    /**
     * This method will save Add/Update Time in actual labors
     */
    _saveTime: function(e) {
        var time = {};
        var self = this;
        var personId = this.$.report_time_person._id;
    	if (this.$.report_times_transtype.value) {
            time.transtype = this.$.report_times_transtype.value;
        }
    	if (this.$.report_times_regularhrs.value) {
            time.regularhrs = this.$.report_times_regularhrs.value;
        }
        if (this.$.report_times_starttime.value) {
        	time.starttime = this.$.report_times_starttime.value;
        }
        if (this.$.report_times_startdate.value) {
            time.startdate = this.$.report_times_startdate.value;
        }
        if (this.$.report_times_finishtime.value) {
            time.finishtime = this.$.report_times_finishtime.value;
        }
        if (this.$.report_times_finishdate.value) {
            time.finishdate = this.$.report_times_finishdate.value;
        }
        if (this.$.report_times_actuals_taskid.value) {
            time.actualstaskid = parseInt(this.$.report_times_actuals_taskid.value);
        }             
        
        if(this.$.reportedwork_time_craft._craftcode){
            time.craft = this.$.reportedwork_time_craft._craftcode;
        }
        time.vendor = this.vendor ? this.vendor : '';
  	  	time.skilllevel = this.skilllevel ? this.skilllevel : '';
        
        if(personId && this.isAuthorized('TECHNICIAN:REPORT_OTHER_LABOR')){
            time.laborcode = personId;
        }
        else if($M.userInfo.labor && !this.isAuthorized('TECHNICIAN:REPORT_OTHER_LABOR')){
        	time.laborcode = $M.userInfo.labor.laborcode;
        }
        if(this.$.report_time_premium){
        	time.premiumpaycode = this.premiumpay;
        }
                
        var revertSaveOperation = function(){
        	//just let overlay go away
        };
        var message = $M.localize('uitext', 'mxapiwodetail','wpnewtimeadded');
        if (this.record) {
            time.href = this.record.href;
        }
    	var performNewTimeSaveOperation = function(overlayNode) {
            self.fire('save-time-card', time);
        };
        var timeCard=this.$.report_time_card;
        $M.createOverlay(timeCard, message, performNewTimeSaveOperation, revertSaveOperation, this.undoIntervalSecs, this.parentElement.parentElement, null);
    },
    /**
     * Toggle DELETE/SAVE button.
     */
    _toggleDeleteSaveButton: function(e) {
    	if(this.byPassUpdate){
    		return;
    	}
    	var dateformat = $M.getMaxauth().whoami.dateformat;
    	var timeformat = $M.getMaxauth().whoami.timeformat;
    	if(this.record){
    		if(this.record.genapprservreceipt){
    			this.editable=false;
    		}else{
    			this.editable=true;
    			if (!this.record.finishtime_localized && !this.record.finishdate_localized && this.record.startdate_localized && this.record.starttime_localized){
    				var self=this;
					self.isDateDurationUpdated=true; 
    				$M.getServerDateTime().then(function(response){
    					if(self.isWorkorderComp){
    						if($M.isMobile()){
        						self.$.report_times_finishdate.value = moment(response.datetime,dateformat.toUpperCase()).format('YYYY-MM-DD');
        						self.record.finishdate_localized = moment(response.date,dateformat.toUpperCase()).format('YYYY-MM-DD');
        						self.$.report_times_finishtime.value = moment(response.time,timeformat).format('HH:mm');
    							self.record.finishtime_localized = moment(response.time,timeformat).format('HH:mm');
    						} else {
        						self.$.report_times_finishdate.value = moment(response.datetime, dateformat.toUpperCase()).format(dateformat.toUpperCase());
        						self.record.finishdate_localized = moment(response.datetime, dateformat.toUpperCase()).format(dateformat.toUpperCase());
        						self.$.report_times_finishtime.value = response.time;
    							self.record.finishtime_localized = response.time;
    						}
    					}
    				}); 
    			}	
    		}
    	}   
    },
    /**
     * To set the values for task id lookup;
     */
    _returnFromLookup: function(e) {
    	if(e.detail.records && e.detail.records[0] && e.detail.records[0].craft && e.detail.records[0].laborcode){
           this.$.report_time_person._id = e.detail.records[0].laborcode;
      	   this.$.report_time_person.value = e.detail.records[0].persondisplayname ? e.detail.records[0].persondisplayname : e.detail.records[0].laborcode;
      	   var personImage = (e.detail.records[0].labor.person._imagelibref)? e.detail.records[0].labor.person._imagelibref : '';
      	   this.$.person_image.image = personImage;
      	   $j(this.$.person_image).css('visibility', personImage?'visible':'hidden');
      	   this.$.reportedwork_time_craft.value = e.detail.records[0].craftdescription;
     	   this.$.reportedwork_time_craft_skilllevel.label = e.detail.records[0].craftskilldescription;
     	   this.craftcode = '';
     	   this.$.reportedwork_time_craft._craftcode = e.detail.records[0].craft.craft;
     	   this.craftcode = e.detail.records[0].craft.craft;
     	   this.skilllevel =  e.detail.records[0].craftskill.skilllevel;
     	   this.craft = '';
           this.craft = e.detail.records[0].craft.craft; 
     	   this.laborcode = '';
     	   this.laborcode = e.detail.records[0].laborcode;
     	   this.vendor = e.detail.records[0].companies.company;
     	   this.hasCraft = (this.$.reportedwork_time_craft.value)?true:false;
        }
    	else if (e.detail.records && e.detail.records.length && e.detail.records[0].taskid) {
        	this.$.report_times_actuals_taskid.value = e.detail.records[0].taskid;
        }else if(e.detail.records && e.detail.records.length && e.detail.records[0].amctcraftdescription){
            this.$.reportedwork_time_craft.value = this.fixDescription(e.detail.records[0].craft,e.detail.records[0].amctcraftdescription);
            this.craftcode = '';
            this.craft = '';
            this.$.reportedwork_time_craft._craftcode = e.detail.records[0].craft;  
            this.craftcode = e.detail.records[0].craft;
            this.craft = this.craftcode;
            this.$.reportedwork_time_craft_skilllevel.label = e.detail.records[0].craftskill.description;
            this.skilllevel =  e.detail.records[0].craftskill.skilllevel;
            this.vendor = e.detail.records[0].companies.company;    
            this.isDirty = true;
            this.hasCraft = true;
       		this.$.report_time_premium.value = '';
       		this.$.report_time_premium_readonly_toggle.value = '';
       		this.premiumpay = '';
       		this.$.reportedwork_time_craft.value = e.detail.records[0].amctcraftdescription;
        }
        else if (e.detail.records && e.detail.records.length && e.detail.records[0].premiumpaycode) {
        	this.premiumpay = e.detail.records[0].premiumpaycode;
        	this.$.report_time_premium.value = (e.detail.records[0].description)?e.detail.records[0].description : e.detail.records[0].premiumpaycode;
        }
    },
    /**
     * Calculate duration between startDate and endDate
     * Update duration time spinner
     */
    _calculateDurations: function(e){
    	if(this.isDirty){
    		this._enableSave();
        	if(this.byPassUpdate || (this.$.report_times_regularhrs && this.$.report_times_regularhrs.value<0)){
        		return;
        	}
        	if(e){
        		e.stopPropagation();
        		e.preventDefault();
        	}
            if (this.isDateDurationUpdated){	
            	var datetimeformat=$M.getMaxauth().whoami.dateformat.toUpperCase()+' '+$M.getMaxauth().whoami.timeformat;
    			var startDate = this.$.report_times_startdate.value;
    			var startTime = this.$.report_times_starttime.value;
    			var endDate = this.$.report_times_finishdate.value;
    			var endTime = this.$.report_times_finishtime.value; 
    			var regularHrs = this.$.report_times_regularhrs.value;
    			if(startDate && startTime && endDate && endTime ){
    				this.isDateDurationUpdated=false;
    				var startDateTime =startDate + ' ' + startTime ;
    				var endDateTime = endDate+ ' ' + endTime ; 
    				
    				if(moment(startDateTime, 'YYYY-MM-DD HH:mm',true).isValid()){
    					startDateTime = moment(startDateTime,'YYYY-MM-DD HH:mm').format(datetimeformat);
    				}
    				
    				if(moment(endDateTime, 'YYYY-MM-DD HH:mm',true).isValid()){
    					endDateTime = moment(endDateTime,'YYYY-MM-DD HH:mm').format(datetimeformat);
    				}
    				
    				var hours = this._getDateDiffInHours(startDateTime , endDateTime);
    			    var startDateTimeMoment = moment(startDateTime,datetimeformat);
    			    var endDateTimeMoment = moment(endDateTime,datetimeformat);

    				if(endDateTimeMoment > startDateTimeMoment){
    					this.$.report_times_regularhrs.value = hours;
    				}else if(hours!==regularHrs){
    					this.$.report_times_regularhrs.value = 0;
    				}
    				this.isDateDurationUpdated=true;
    			}
    	    }
    	}
	},
	/**
     * Return difference in hours between two dates
     */	
	_getDateDiffInHours: function(startDateTime , endDateTime){
		var datetimeformat=$M.getMaxauth().whoami.dateformat.toUpperCase()+' '+$M.getMaxauth().whoami.timeformat;
		var durationMS = moment(endDateTime,datetimeformat).diff(moment(startDateTime,datetimeformat));
		var hours = this.convertMstoHM(durationMS);
        return hours;
    },
	/**
     * Updating endDateTime on change in regular hours
     */	
    _updateEndDate: function(e){
    	if(this.isDirty){
    		if(this.byPassUpdate){
        		return;
        	}
        	this._enableSave();
        	e.stopPropagation();
        	e.preventDefault();
        	if(this.isDateDurationUpdated){
    	    	var startDate = this.$.report_times_startdate.value;
    			var startTime =this.$.report_times_starttime.value;
    			var regularHrs = this.$.report_times_regularhrs.value;
    			var dateformat = $M.getMaxauth().whoami.dateformat.toUpperCase();
    			var timeformat = $M.getMaxauth().whoami.timeformat.toLowerCase();
    			if(startDate && startTime && regularHrs >= 0){
    				var datetimeformat=$M.getMaxauth().whoami.dateformat.toUpperCase()+' '+$M.getMaxauth().whoami.timeformat;
    				var tempDate = '';
    				if($M.isMobile()){
    					tempDate = moment(moment(startDate,'YYYY-MM-DD').format(dateformat) + ' ' + startTime,datetimeformat).format(datetimeformat);
    				} else {
    					tempDate = moment((moment(startDate,dateformat).format(dateformat) + ' ' + startTime),datetimeformat).format(datetimeformat);	
    				}
    				var startDateTime=moment(tempDate,datetimeformat).add(moment.duration(Formatter['HH:MM'](e.target.value)).asMinutes(),'minutes');
    				this.isDateDurationUpdated=false;
    				if(!isNaN(Date.parse(startDateTime)) && moment(startDateTime).format(timeformat) !== this.$.report_times_finishtime.value){
    		    		if (timeformat.includes('a') && !$M.isMobile()){
    						this.$.report_times_finishtime.value = moment(startDateTime).format('hh:mm A'); 
    		    		}else{
    						this.$.report_times_finishtime.value = moment(startDateTime).format('HH:mm'); 
    		    		}
    		    	} 
    				if(!isNaN(Date.parse(startDateTime)) && moment(startDateTime).format(dateformat) !== this.$.report_times_finishdate.value){
    					if($M.isMobile()){
    						this.$.report_times_finishdate.value = moment(startDateTime, datetimeformat).format('YYYY-MM-DD'); 
    					} else {
    						this.$.report_times_finishdate.value = moment(startDateTime).format(dateformat); 
    					}
    		        } else if($M.isMobile()){
    					this.$.report_times_finishdate.value = moment(startDateTime, datetimeformat).format('YYYY-MM-DD'); 
    		        }
    				this.isDateDurationUpdated=true;
    			}
        	}
    	}
     },
    /**
	  * Enable/disable save button for required report time card fields.
	  */
   _enableSave : function(){
	   if(this.isDirty){
		   this.async(function(){
	   			 var craftcode,startdate,transtype,labor;
	   			 startdate = this.$.report_times_startdate.value;
	   			 transtype = this.$.report_times_transtype.value;
				 craftcode = this.$.reportedwork_time_craft.value;
				 laborcode = this.$.report_time_person.value;
				 this.isDateDurationUpdated = true;
				 if(laborcode && startdate && transtype && craftcode && this.isAuthorized('TECHNICIAN:REPORT_OTHER_LABOR')){
					 $j(this.$.saveReportTimeCardDiv).find('*').prop('disabled',false);
				 } 
				 else if(startdate && transtype && craftcode && !this.isAuthorized('TECHNICIAN:REPORT_OTHER_LABOR')){
					 $j(this.$.saveReportTimeCardDiv).find('*').prop('disabled',false);
				 }else{
					 $j(this.$.saveReportTimeCardDiv).find('*').prop('disabled',true);
				 }
		   	},100); 	
		   	if(this.$.reportedwork_time_craft && !this.$.reportedwork_time_craft.value){
				this.craft = '';
				this.$.reportedwork_time_craft_skilllevel.label = '';
				this.$.reportedwork_time_craft._craftcode = '';
				this.hasCraft = false;
				this.$.report_time_premium.value = '';
				this.$.report_time_premium_readonly_toggle.value = '';
				this.premiumpay = '';
				this.craftcode = '';
			}else{
				this.hasCraft = true;
				this.craftcode = '';
				this.craftcode = this.$.reportedwork_time_craft._craftcode;
			}
		   	if(!this.$.report_time_person.value){
				this.laborcode = '';
				this.$.person_image.image = '';
				this.$.report_time_person._id = '';
			}
			if(this.$.report_time_premium && !this.$.report_time_premium.value){
				this.premiumpay = '';
			}
	   }
   },
   /** 
    *  When editing a technician actual times then all others tasks must be disable. 
    */ 
    _disablelist: function(e) { 
        var selectedElement = '#'+ this.id;
        var addTimeDiv = $M.workScape.querySelector('#techReportWork_addTime');
        var reportWorkSelector = $M.workScape.querySelector('#techReportWork_technicianReportWorkSelector');
        var mainPage = $M.workScape.querySelector('#techReportWork_mainPage');
        //Disable set of elements
        $M.toggleDisabled([addTimeDiv, reportWorkSelector, mainPage], this.isDirty && this.id);
        if(this.isDirty && this.id){ 
             $M.toggleDisabled($j('#techReportWork_reportedTimeList').children().not(selectedElement).toArray(), true);
             $M.allowPageNavigation(false);
         } else {
             $M.toggleDisabled($j('#techReportWork_reportedTimeList').children().toArray(), false);
             $M.allowPageNavigation(true);
         }
	},
	
	/**
	 * Convert milliseconds to hh:mm format
	 */
	convertMstoHM : function ( ms ) {
	    var seconds = ms / 1000;
	    var hours = parseInt( seconds / 3600 );
	    seconds = seconds % 3600;
	    var minutes = parseInt( seconds / 60 );
	    seconds = seconds % 60;
	    return hours+':'+minutes;
	},
	
	/**
	 * @param crafts:- list of craft for the loggedin labor.
	 * @return:- craft list for current logged in labor.
	 */
	_craftList : function(crafts){
		var externalValues=[];
		crafts.forEach(function(member){
			externalValues.push(member.$alias_this_attr$craft+':'+(member.craft.description?member.craft.description:member.$alias_this_attr$craft));
		});
		if (crafts.length===1 && this.editable){
			$j(this.$.reportedwork_time_craft).attr('','visibility', 'hidden');
		}
		return externalValues.toString();
	},	
	
	/**
	 * @param regularhrs:- Regular Hours for the labor transaction.
	 * format decimal value in text in format (like 1 Hours 30 Minutes).
	 */
	_formatRegularHours : function(regularhrs){
		this.async(function(){ 
				var duration=moment.duration(regularhrs,'hours');
				var hoursLocalized;
				var minutesLocalized;
				
				if(duration._data.hours > 1 || Math.abs(duration._data.hours) > 1){
					hoursLocalized = $M.localize('uitext', 'mxapiwodetail', 'Hours');
				}else{
					hoursLocalized = $M.localize('uitext', 'mxapibase', 'TimeHR');
				}
				
				if(duration._data.minutes > 1 || Math.abs(duration._data.minutes) > 1){
					minutesLocalized = $M.localize('uitext', 'mxapiwodetail', 'Minutes');
				}else{
					minutesLocalized = $M.localize('uitext', 'mxapibase', 'TimeMIN');
				}
				var hours= duration._data.hours+24*duration._data.days;
				var minutes= duration._data.minutes;
				if(regularhrs<0){
					hours = -Math.abs(hours);
					minutes = Math.abs(minutes);
				}
				var html;
				if(regularhrs<0 && hours===0){
					html= '-'+hours +' '+ '<span style="font-size:28px;">' + hoursLocalized + '</span>'+' '+ (duration._data.seconds===59?minutes+=1:minutes) +' '+ '<span style="font-size:28px;">' + minutesLocalized + '</span>';
				}else{
					html= hours +' '+ '<span style="font-size:28px;">' + hoursLocalized + '</span>'+' '+ (duration._data.seconds===59?minutes+=1:minutes) +' '+ '<span style="font-size:28px;">' + minutesLocalized + '</span>';
				}
				
				$j(this).find('.report_times_hours_label_spinner > label').html(html);
		},100); 
	},
	/*
     * Finish date/time format.
     */
	_formatFinishDate : function(record){  
		var finishDate = record.finishdate_localized;
		var finishTime = record.finishtime_localized;
		var html = '';
		if(finishDate){
			html = finishDate;
		}
		if(finishTime){
			html += ' - ' + finishTime;
		}		
		
		return html;
	},
	/*
     * Start date/time format.
     */
	_formatStartDate : function(record){  
		var startDate = record.startdate_localized;
		var startTime = record.starttime_localized;
		var html = '';
		if(startDate){
			html = startDate;
		}
		if(startTime){
			html += ' - ' + startTime;
		}		
		
		return html;
	},
	/*
	 * Get craft for labor Lookup.
	 */
	_getCraftClause : function(){           
	     var craftClause = '';
	     var allowanycraft = sessionStorage.getItem('allowanycraft');      
	     if(allowanycraft === '0'){
	        craftClause =  'oslc.where=amctcraft.laborcraftrate.laborcode=%22'+ $M.getUserInfo().labor.laborcode +'%22%20and%20orgid=%22'+ $M.userInfo.defaultOrg + '%22';
	        if(this.isAuthorized('TECHNICIAN:REPORT_OTHER_LABOR')){
	        	if(this.laborcode){
	        		craftClause =  'oslc.where=amctcraft.laborcraftrate.laborcode=%22'+ this.laborcode +'%22%20and%20orgid=%22'+ $M.userInfo.defaultOrg + '%22';
	        		if(this.vendor){
		        		craftClause +='%20and%20companies.company=%22'+ this.vendor +'%22';
				    }
	        	}
	        	else{
	        		craftClause =  'oslc.where=orgid=%22'+ $M.userInfo.defaultOrg + '%22';
	        	} 
			} 
	     }
	     else {
	        craftClause =  'oslc.where=orgid=%22'+ $M.userInfo.defaultOrg + '%22';
	     }
	    
	     return craftClause;
	},
	/**
	 * Show id if description is unspecified
	 */
    _validateDesc: function(record,indentifier){
		if(record && record.craft.craft  && indentifier==='craft'){
			return this.fixDescription(record.craft.craft,record.craft.description);
		}else if(record && record.transtype && indentifier==='type'){
			return this.fixDescription(record.transtype,record.transtype_description);
		}
    },
	/*
    * Get laborclause for labor Lookup.
	*/
   _getLaborClause : function(craft){
	   var tempLaborClause = '';
 	    var allowanycraft = sessionStorage.getItem('allowanycraft');
 	    if(allowanycraft === '0') {
	 	    if(craft){
	 	    	tempLaborClause = 'oslc.where=craft.craft=%22'+ this.craft + '%22%20and%20orgid=%22'+ $M.userInfo.defaultOrg + '%22';
			}
	 	   if(craft && this.vendor){
		    	tempLaborClause +='%20and%20companies.company=%22'+ this.vendor +'%22';
		    }
 	    }
 	    if(!tempLaborClause){
 	    	tempLaborClause = 'oslc.where=orgid=%22'+ $M.userInfo.defaultOrg + '%22';
	    } 	
		return this.laborClasue = tempLaborClause;
	},
	/*
	 * Get premium for craft Lookup.
	 */
	_getPremiumClause : function(craftcode){
		var premiumClause='';
		if(craftcode){
			premiumClause = "setupdata={\'labtrans\':{\'craft\':\'"+craftcode+"\'}}";
		}
		return premiumClause;
	},
	/*
	 * Get sigoption for premium code.
	 */
	_getPremiumCodeSigOption : function(record){
		if(this.isAuthorized('TECHNICIAN:PREMIUMCODE')){
			return false;
		}
		return true;
	},
	/*
	 * Get sigoption for labor.
	 */
	_getLaborSigOption : function(record){
		if(this.isAuthorized('TECHNICIAN:REPORT_OTHER_LABOR')){
			return false;
		}
		return true;
	},
	/*
	 * Get craft skill description
	 */
	_getCraftSkillDescription : function(record){
		var craftDescription;
		if(record) {
			if(record.craftskill.length === 1 || record.craftskill[0].skilllevel){
				craftDescription = record.craftskill[0].description;
			}else if(record.craftskill.length > 1) {
				craftDescription = record.craftskill[1].description;
			}
		}
		return craftDescription;
	},
	/*
	 * Get pemiumpay code description
	 */
	_getPremiumPayCodeDesc : function(record){
		return (record.premiumpaycodedesc)? record.premiumpaycodedesc : record.premiumpaycode;
	},
	/*
	 * Get labor code if person display name not availble.
	 */
	_getLaborInfo : function(record){
		var laborcode='';
		if(record){
			laborcode = record.person.displayname ? record.person.displayname : record.laborcode;
		}
		return laborcode;
	}
	
});
