/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2017,2018
 */
Polymer({
	is: 'maximo-attribute-lookup',
  	behaviors: [BaseComponent, DesignBehavior,HTMLEncoder],
  	listeners: {'chooseObject' : '_chooseObject',
  				'maximo-checkbox-changed' : '_checkboxChanged',
  				'schema-data-refreshed' : '_setupSchemaData',
  				'relationshipSelect' : '_relationshipSelect',
  				'fieldSelected' : '_fieldSelected',
  				'showInfo' : '_showInfo',
  				'designer-field-changed' : '_designFieldChanged',
  				'designer-field-changed-drop' : '_designFieldDropped',
  				'designer-save-completed' : '_saveCompleted',
  				'designer-attributes-changed' : '_updateAttributeList'},
  	properties : {
  		_changes: {
  			type: Object,
  			value: function(){
  				return {};
  			}
  		},
    	selectedObj : {
    		type: Object,
    		value: function(){
    			return {};
    		}
    	},
    	schemaInfo : {
    		type: Object,
    		value: null
    	},
    	relationship : {
    		type : String,
    		value: null
    	},
    	_intObjectName:{
    		type : String,
    		value: 'MXAPIWODETAIL'
    	},
    	_excludeAttrList: {
    		type: Array,
    		value: function(){
    			return ['_id','_rowstamp','href','localref'];
    		}
    	}
  	},
  	/**
     * Close dialog 
     */
  	_closeDialog: function(){
		$M.closeDialog();
  	},
  	
  	/**
  	 * Setup data on initial load of component
  	 */
  	attached:function(){
  		var self = this;
  		
  		//append card into card design dialog
  		if(this.domHost && this.domHost.data){
	  		this.$.cardtemplate.append(this.domHost.data);
	  		
		  	this.$.cardtemplate.addEventListener('mousedown', function(ev){
		    	
			    var topComponentElement = self.getConfigurableParent(ev.target);
				var childComponent = topComponentElement.getMyComponent();
			    
				var noLabelDrop = childComponent.tagName === 'MAXIMO-LABEL' && $j(childComponent).attr('drop-enabled') === 'false';
			    var readonly = (!noLabelDrop|| childComponent.tagName === 'MAXIMO-IMGLIB') && childComponent.tagName !== 'MAXIMO-FIELD';
			    
			    if(topComponentElement && childComponent && childComponent._propertySheetAttributes){
					$M.showDialog(self, 'propertySheetID', $M.localize('uitext','mxapibase','configureContent'), 'maximo-edit-propertydialog', false, null, {'datasetRecord':topComponentElement, 'readonly':readonly});
			    }
		  	}, true);
		  	
		  	this.$.cardtemplate.firstElementChild.className = 'designMode '+ this.$.cardtemplate.firstElementChild.className;
		  	this.async(function(){
		  		$j(this.$.cardtemplate).find('maximo-component').each(function(){
		  			var maxComponent = this.getMyComponent();
		  			if(maxComponent && maxComponent._propertySheetAttributes){
		  				$j(maxComponent).toggleClass('editmode',true);
		  			}
		  		});
		  	}, 200);
  		}
	
  		//TODO - set intObjectName from collection, currently defaults to MXAPIWODETAIL
  		self.selectedObj = {'intobjectname':this._intObjectName};
  		  		
  		this.$.schemacollection.refreshRecords().then(function(data){
  			self.$.schemacollection.schemaUrl = data.response.$ref;
  			self.$.schemacollection.collectionUri = data.response.$ref;
  			self.selectedObj.fullRecord = data.response;
  			self._setupObjectDatalist();
  		});
  			
  	},
  		
  	/**
  	 * When object is selected from the datalist, the relationship and fields will be updated to reflect selection.
  	 */
  	_chooseObject : function (e) {
		var self = this;
				
		if (e.detail._selectedRecords && (e.detail._selectedRecords.length > 0)) {
			this.selectedObj=e.detail._selectedRecords[0];
			this._updateListOfRelationships();
			
			if(this.selectedObj.href){ //jsonmbo (child object)
				this.$.schemacollection.collectionUri = this.selectedObj.href;
				this.$.schemacollection.schemaUrl = this.selectedObj.href;
			} else if(this.selectedObj.fullRecord.$ref){ //jsonschema (main OS record)
				this.$.schemacollection.collectionUri = this.selectedObj.fullRecord.$ref;
				this.$.schemacollection.schemaUrl = this.selectedObj.fullRecord.$ref;
			}
			
			this.$.schemacollection.refreshRecords().then(function(dataSchema){
	  			self.selectedObj.fullRecord = dataSchema.response;
	  			self._updateListOfAttributes();	
	  		});	
		}
	},
  	
	
	/**
	 * Called when we have schema data.  
	 * Build tree containing, objects and first level relations
	 */
	_setupObjectDatalist  : function (e) {
		this.$.objectList.toggleBlur(true);
		
		this.async(function(){
			this.objectdatalist = [];
			var dataList = this._buildObjectTree(this.schemaInfo);
			if (dataList) {
				this.objectdatalist.push(dataList);
			}			
			
			this.$.objectList.refresh();
		
		},100);
	},
	
	/**
	 * takes the schemainfo object and builds a simpler tree out of it suitable for displaying on the datalist.
	 */
	_buildObjectTree : function(parent) {
		if ((parent) && ((parent.type === 'array') || (parent.type === 'object')))
		{
			var record = {};
			record.fullRecord = parent;
			record.description = parent.title;
			
			var childlist = [];
			Object.keys(parent.relations).map(function(k) {
				var attr = parent.relations[k];
				
				//filter out child objects if no object doesnt contain any SINGLE cardinality relationships.
				var relationships = attr.relations;
				for(var rel in relationships){
					var cardinality =  relationships[rel].cardinality; 
					if(cardinality==='SINGLE'){
						attr.description = k;
						attr.parent = parent.title;
						childlist.push(attr);
						break;
					} else {
						continue;
					}
				}
			});
			
			if (childlist.length > 0) {
				childlist.sort(function(a,b) {
		  			return (a.description.localeCompare(b.description));
		  		});
				record.children = childlist;
			}
			
			return record;
		}
		return null;
	},
	
	/**
	 * Prepare data to display in Relationships datalist when object is selected and prepare info icon data.
	 */
	_updateListOfRelationships : function(){
		this.$.relationshiplist.toggleBlur(true);
		var self = this;
		this.async(function(){
			if (this.selectedObj) {
				var relArray = [];
				var relationshipList = this.selectedObj.relations;
				this.$.relationshiplist.clearSelected();
				
				for(var rel in relationshipList){
					var cardinality = relationshipList[rel].cardinality; 
					if(cardinality!=='SINGLE'){
						//only display SINGLE cardinality relationships
						continue;
					}
					var relObj = {};
					relObj.relationship = true;
					relObj.description = relationshipList[rel].name;
					relObj.whereclause = relationshipList[rel].whereclause;
					relObj.cardinality = cardinality;
					relObj.parent =this.selectedObj.parent;
					relObj.child = this.selectedObj.description;
					relArray.push(relObj);
				}
				
				if (relArray.length > 0) {
					relArray.sort(function(a,b) {
			  			return (a.description.localeCompare(b.description));
			  		});
				}
				
				self.relationshipDataList = relArray;
				this.$.relationshiplist.toggleBlur(false);
			}
		});
	},
	
	/**
	 * Setup attributes to display in the fields datalist.
	 */
	_updateListOfAttributes : function () {
		this.$.fieldList.toggleBlur(true);
		var attributeLookup = this;
		this.async(function(){
			var objectdescription = '';
			var fields = {};
			this.requiredMap = new Map();
			var self = this;
			
			if (this.selectedObj) {				
				if (this.selectedObj.fullRecord) {
					if (this.selectedObj.fullRecord.properties) {
						fields=this.selectedObj.fullRecord.properties;
					}
					
					if (this.selectedObj.fullRecord.required) {
						this.selectedObj.fullRecord.required.forEach(function(item) {
							self.requiredMap.set(item,item);
						});
					}
				}
			}

			var requiredOnly = this.$.requireFieldSwitch.checked;
			var temp = [];
			var searchFieldValue = $j(this.$.searchfields).find('input')[0].value;
			
			Object.keys(fields).map(function(k) {
				var attr = fields[k];
				if(self._excludeAttrList.includes(k)){
					return;
				}
				// note: if the attribute type is array, then that is child tree node.
				// filter that out for now until we decide to support showing/filtering on the full tree.
				if (attr.type && attr.type !=='array') {
					if ((!requiredOnly) || (requiredOnly && self.requiredMap.has(k))) {
						
						//if search field is defined we need to define imglibref special here since its not part of the original collection.
						if(k==='_imglibref'){
							if(searchFieldValue && !'_imglibref'.includes(searchFieldValue)){
								//if search value is not defined
								return;
							}
						}
						attr.attribute = k;
						attr.upperattribute = attr.attribute.toUpperCase();
						if ((attr.title === undefined) || (attr.title === null)) {
							attr.title = attr.upperattribute;
						}
						attr.upperdescription = attr.title.toUpperCase();
						attr.datalistVisible = true;
						attr.href = self.uniqueid+'_'+attr.attribute;
						attr.objectdescription = objectdescription + attr.upperattribute;
						if(attributeLookup.selectedObj){
							attr.object = attributeLookup.selectedObj.fullRecord.resource;
						}
						
						temp.push(attr);
					}
				}
			});

			temp.sort(function(a,b) {
				return a.attribute.localeCompare(b.attribute);
			});

			this.set('fieldDataList',temp);
			
		},300);
	},
		
	/**
	 * Show relationship / field tooltip
	 */ 
	_showInfo: function(e){
		var infoButton = $j(e.detail.item).find('button[originalEvent=info]');
		var availableItem=e.detail.record;
		if (availableItem){
			var tooltip = document.createElement('div');

			if (availableItem.relationship===true){ //bind and show relationship tooltip
				var relationship=document.createElement('maximo-field');
				var parent=document.createElement('maximo-field');
				var child=document.createElement('maximo-field');
				var whereclause=document.createElement('maximo-field');
				var cardinality=document.createElement('maximo-field');
				
				$j(relationship).append('<br>');
				$j(parent).append('<br>');
				$j(child).append('<br>');
				$j(whereclause).append('<br>');
				
				$j(relationship).attr({'label': $M.localize('uitext', 'mxapibase', 'relationships'),'datalabel':HTMLEncoder.sanitize(availableItem.description)});
				$j(parent).attr({'label': $M.localize('uitext', 'mxapibase', 'parent'),'datalabel':HTMLEncoder.sanitize(availableItem.parent)});
				$j(child).attr({'label': $M.localize('uitext', 'mxapibase', 'child'),'datalabel':HTMLEncoder.sanitize(availableItem.child)});
				$j(whereclause).attr({'label': $M.localize('uitext', 'mxapibase', 'whereclause'),'datalabel':HTMLEncoder.sanitize(availableItem.whereclause),'wrapper-class':'wrap'});
				$j(cardinality).attr({'label': $M.localize('uitext', 'mxapibase', 'cardinality'),'datalabel':HTMLEncoder.sanitize(availableItem.cardinality)});
				$j(tooltip).append(relationship,parent,child,whereclause,cardinality);
				
			} else { //bind and show field tooltip
				var fieldname=document.createElement('maximo-field');
				var remarks=document.createElement('maximo-field');
				var title=document.createElement('maximo-field');
				var required=document.createElement('maximo-field');
				var maxtype=document.createElement('maximo-field');
				var length=document.createElement('maximo-field');
				
				$j(fieldname).append('<br>');
				$j(remarks).append('<br>');
				$j(title).append('<br>');
				$j(required).append('<br>');
				$j(maxtype).append('<br>');
				
				$j(fieldname).attr({'label': $M.localize('uitext', 'mxapibase', 'fieldname'),'datalabel':HTMLEncoder.sanitize(availableItem.title)});
				$j(remarks).attr({'label':  $M.localize('uitext', 'mxapiwodetail', 'remarks'),'datalabel':(availableItem.remarks)});
				$j(title).attr({'label':  $M.localize('uitext', 'mxapibase', 'title'),'datalabel':HTMLEncoder.sanitize(availableItem.title)});
				$j(required).attr({'label': $M.localize('uitext', 'mxapibase', 'requiredfield'),'datalabel':this.requiredMap.has(HTMLEncoder.sanitize(e.detail.record.attribute))});
				$j(maxtype).attr({'label':$M.localize('uitext', 'mxapibase', 'databasetype'),'datalabel':HTMLEncoder.sanitize(availableItem.subType)});
				$j(length).attr({'label': $M.localize('uitext', 'mxapibase', 'fieldlength'),'datalabel':HTMLEncoder.sanitize(availableItem.maxLength)});
				$j(tooltip).append(fieldname,remarks,title,required,maxtype,length);
			}
			
			$M.showTooltip(tooltip,infoButton[0]);
		}
	},
	
	/**
	 * Checkbox toggle to refresh and show/hide required fields.
	 */
	_checkboxChanged : function(e) {
		var self = this;
		if (e.target === this.$.requireFieldSwitch) {
			self._updateListOfAttributes();	
		}
	},
	
	/**
	 * Set selected relationship
	 */
	_relationshipSelect : function(e){
		if(e.target._selectedRecords.length>0){
			this.relationship = e.target._selectedRecords[0].description;
		}
	},
	
	/**
	 * Field selected
	 */
	_fieldSelected : function(e){
		if(e.target._selectedRecords.length>0){
			console.log(e.target._selectedRecords[0].upperattribute);
			console.log('Related link:  '+this.relationship+'.'+e.target._selectedRecords[0].upperattribute);
		}
	},
	
	/**
	 * Schema data setup.
	 * Load record results into object and update field datalist.
	 */
	_setupSchemaData : function(response){
		if(response && response.detail){
			if(!this.selectedObj){
				this.selectedObj = {};
			}
			this.selectedObj.fullRecord = response.detail;
			this._updateListOfAttributes();	
		}
	},
	
	_designFieldDropped: function(e){
		var dropAttributes = e.detail.attributes;
		var component = e.detail.component;
		var attributes = {};
	
		var valueAttribute = 'value';
		if(component.getMyComponent().tagName === 'MAXIMO-LABEL'){
			valueAttribute = 'label';
		}
		
		if(dropAttributes.title && valueAttribute === 'value'){
			attributes.label = dropAttributes.title;
			attributes.localize = {'label':dropAttributes.title};
		}
		
		var relationship = '';
		if(this.relationship){
			if(this.selectedObj.relations){
				relationship = '.'+this.relationship;
			}
			else{ //clear out value as no relationship exists
				this.relationship = ''; 
			}		
		}
		// add '_localized' to all attributes of type DATE, TIME or DATETIME
		var attributeType = e.detail.attributes.subType;
		var localizedRepo = true;
		var i = 0;
		while(i < this.collections.length){
			if(this.collections[i].localizedRepo !== '1'){//if one collection cannot be localized, localization cannot be done as same value will appear in the case of multiple collections
				localizedRepo = false;
				break;
			}
			i++;
		}
		if(localizedRepo === true){
			attributes[valueAttribute] = attributeType === 'DATE' || attributeType === 'TIME' || attributeType === 'DATETIME' || attributeType === 'DURATION' || attributeType === 'AMOUNT' ? '{{record'+relationship.toLowerCase()+'.'+dropAttributes.attribute+'_localized'+'}}' : '{{record'+relationship.toLowerCase()+'.'+dropAttributes.attribute+'}}';			
		}
		else{
			attributes[valueAttribute] = '{{record'+relationship.toLowerCase()+'.'+dropAttributes.attribute+'}}';
		}			
		
		var attribute = relationship.length > 0 ? relationship.substring(1).toLowerCase()+'.'+dropAttributes.attribute : dropAttributes.attribute;		
		var replaceAttribute;
		var unboundAttributes = JSON.parse(e.detail.component.getMyComponent().getAttribute('unbound-designer-attributes'));
		if(unboundAttributes){
			replaceAttribute = unboundAttributes[valueAttribute];
			//remove the underscore and text after for the following case, these attributes appear in the attribute-names list without '_localized'
			if(replaceAttribute.indexOf('_localized') !== -1){
				replaceAttribute = replaceAttribute.substring(0,replaceAttribute.indexOf('_localized'));
			}
		}
		var attributeListChange = {'oldAttribute': replaceAttribute,'newAttribute': attribute};
		this.fire('designer-attributes-changed',attributeListChange);
			
		var change = {'component':component, 'attributes': attributes};
		this._changes[component.getAttribute('cid')] = change;
		this.fire('designer-update-components', [change]);
		e.stopPropagation();
	},
	
	_designFieldChanged: function(e){			
		var change = {'component':e.detail.component, 'attributes': e.detail.attributes};
		this._changes[change.component.getAttribute('cid')] = change;
		this.fire('designer-update-components', [change]);
		e.stopPropagation();
	},
	
	/**
	 * Apply the _changes array by sending to the server
	 */
	_save: function(){
		$M.toggleWait(true);
		this.fire('designer-create-delta', Object.values(this._changes));
		// we will need to allow for a callback to close dialog once the update succeeds
	},
	
	/**
	 * Should be called via event on save event.details is null on success and 
	 */
	_saveCompleted: function(e){
		e.stopPropagation();
		var changes = Object.values(this._changes);
		changes.forEach(function(change, index){
			delete changes[index].componentUiId;
		});
		this.fire('designer-update-components-on-save', changes);
		this._closeDialog();
		$M.toggleWait(false);
	},
	
	/**
	 * Should be called via event when a new field is dropped onto the card or when the user edits a card field that only has a label
	 * Adds new attribute to the attribute-names list of every collection involved and removes the old attribute from the attribute-names list
	 */
	_updateAttributeList: function(e){
		var replaceAttribute = e.detail.oldAttribute;
		var attribute = e.detail.newAttribute;
		var collectionAttributes = {};
		
		this.collections.forEach(function(collection){
			collection.designerNames = collection.designerNames ? collection.designerNames+','+attribute : attribute;
			var modifiedList = this._removeAttribute(collection.designerNames,replaceAttribute);
			if(modifiedList.length !== collection.designerNames.length){//an attribute has been removed from designerNames
				collection.designerNames = modifiedList;
				collection.attributeNames = this._removeAttribute(collection.attributeNames,replaceAttribute);
				collection.searchAttributeNames = this._removeAttribute(collection.searchAttributeNames,replaceAttribute);
				collection.searchAttributeNames = this._removeNonSearchableAttribue(collection.searchAttributeNames);
			}
			collectionAttributes.attributeNames = collection._designerAttributeNames;
			collectionAttributes.searchAttributeNames = this._removeNonSearchableAttribue(collection._designerSearchAttributeNames);
			collectionAttributes.designerNames = collection.designerNames;
			var component = $M.findElementParent(collection, 'MAXIMO-COMPONENT');
			this._changes[component.getAttribute('cid')] = {'component':component, 'attributes' : collectionAttributes};
		}.bind(this));
	},
	
	/**
	 * Removes the attribute that has been replaced on the card from the attribute-names list only if it was a new attribute that was not
	 * originally on the collection.
	 * @param attributeList - the attribute-names list that contains all attributes listed on the collection
	 * @param attribute - the attribute to be removed from the attribute-names list
	 * @return modifiedList- the attribute-names list without the old attribute
	 */
	_removeAttribute: function(attributeList,attribute){
		var modifiedList;
		var index = -1;
		if((index = attributeList.indexOf(','+attribute+',')) != -1){//attribute in middle of list
			modifiedList = attributeList.substring(0,index) + attributeList.substring(index + attribute.length+1);
		}
		else if(attributeList.indexOf(','+attribute) != -1){//attribute at end of list
			var endIndex = attributeList.length - (attribute.length + 1);
			modifiedList = attributeList.substring(0,endIndex);
		}
		else if(attributeList.indexOf(attribute+',') != -1){//attribute at beginning of list
			var startIndex = attribute.length + 1;
			modifiedList = attributeList.substring(startIndex);
		}
		else{//attribute not found
			modifiedList = attributeList;
		}
		
		return modifiedList;
	},
	
	/**
	 * remove imglibref,imagelibrif from search attributes here since its not part of the original collection.						
	 */
	_removeNonSearchableAttribue: function(attributeList){
		if(attributeList) {
			var attributesArr = attributeList.split(',');
			return attributesArr.filter(attribute => !attribute.includes('_imglibref') && !attribute.includes('_imagelibref')).toString();
			}
		return attributeList;
	}
});